# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

"""Low-level access to LAPACK functions. """

__all__ = ['sgetrf', 'dgetrf', 'cgetrf', 'zgetrf',
           'sgetrs', 'dgetrs', 'cgetrs', 'zgetrs',
           'sgecon', 'dgecon', 'cgecon', 'zgecon',
           'sggev', 'dggev', 'cggev', 'zggev',
           'sgees', 'dgees', 'cgees', 'zgees',
           'strsen', 'dtrsen', 'ctrsen', 'ztrsen',
           'strevc', 'dtrevc', 'ctrevc', 'ztrevc',
           'sgges', 'dgges', 'cgges', 'zgges',
           'stgsen', 'dtgsen', 'ctgsen', 'ztgsen',
           'stgevc', 'dtgevc', 'ctgevc', 'ztgevc',
           'prepare_for_lapack']

import numpy as np
cimport numpy as np

# Strangely absolute imports from kwant.linalg.f_lapack don't work here.  So
# continue using traditional implicit relative imports.
import f_lapack
cimport f_lapack
from f_lapack cimport l_int, l_logical

int_dtype = f_lapack.l_int_dtype
logical_dtype = f_lapack.l_logical_dtype

# exceptions

class LinAlgError(RuntimeError):
    pass


# some helper functions
def filter_args(select, args):
    return tuple([arg for sel, arg in zip(select, args) if sel])

def assert_fortran_mat(*mats):
    # This is a workaround for a bug in NumPy version < 2.0,
    # where 1x1 matrices do not have the F_Contiguous flag set correctly.
    for mat in mats:
        if (mat is not None and (mat.shape[0] > 1 or mat.shape[1] > 1) and
            not mat.flags["F_CONTIGUOUS"]):
            raise ValueError("Input matrix must be Fortran contiguous")


# Wrappers for xGETRF
def sgetrf(np.ndarray[np.float32_t, ndim=2] A):
    cdef l_int M, N, info
    cdef np.ndarray[l_int, ndim=1] ipiv

    assert_fortran_mat(A)

    M = A.shape[0]
    N = A.shape[1]
    ipiv = np.empty(min(M,N), dtype = f_lapack.l_int_dtype)

    f_lapack.sgetrf_(&M, &N, <float *>A.data, &M,
                     <l_int *>ipiv.data, &info)

    assert info >= 0, "Argument error in sgetrf"

    return (A, ipiv, info > 0 or M != N)

def dgetrf(np.ndarray[np.float64_t, ndim=2] A):
    cdef l_int M, N, info
    cdef np.ndarray[l_int, ndim=1] ipiv

    assert_fortran_mat(A)

    M = A.shape[0]
    N = A.shape[1]
    ipiv = np.empty(min(M,N), dtype = f_lapack.l_int_dtype)

    f_lapack.dgetrf_(&M, &N, <double *>A.data, &M,
                     <l_int *>ipiv.data, &info)

    assert info >= 0, "Argument error in dgetrf"

    return (A, ipiv, info > 0 or M != N)

def cgetrf(np.ndarray[np.complex64_t, ndim=2] A):
    cdef l_int M, N, info
    cdef np.ndarray[l_int, ndim=1] ipiv

    assert_fortran_mat(A)

    M = A.shape[0]
    N = A.shape[1]
    ipiv = np.empty(min(M,N), dtype = f_lapack.l_int_dtype)

    f_lapack.cgetrf_(&M, &N, <float complex *>A.data, &M,
                     <l_int *>ipiv.data, &info)

    assert info >= 0, "Argument error in cgetrf"

    return (A, ipiv, info > 0 or M != N)

def zgetrf(np.ndarray[np.complex128_t, ndim=2] A):
    cdef l_int M, N, info
    cdef np.ndarray[l_int, ndim=1] ipiv

    assert_fortran_mat(A)

    M = A.shape[0]
    N = A.shape[1]
    ipiv = np.empty(min(M,N), dtype = f_lapack.l_int_dtype)

    f_lapack.zgetrf_(&M, &N, <double complex *>A.data, &M,
                     <l_int *>ipiv.data, &info)

    assert info >= 0, "Argument error in zgetrf"

    return (A, ipiv, info > 0 or M != N)

# Wrappers for xGETRS

def sgetrs(np.ndarray[np.float32_t, ndim=2] LU,
           np.ndarray[l_int, ndim=1] IPIV, B):
    cdef l_int N, NRHS, info
    cdef np.ndarray b

    assert_fortran_mat(LU)

    # again: workaround for 1x1-Fortran bug in NumPy < v2.0
    if (not isinstance(B, np.ndarray) or
        (B.ndim == 2 and (B.shape[0] > 1 or B.shape[1] > 1) and
         not B.flags["F_CONTIGUOUS"])):
        raise ValueError("In dgetrs: B must be a Fortran ordered NumPy array")

    b = B
    N = LU.shape[0]
    if b.ndim == 1:
        NRHS = 1
    elif b.ndim == 2:
        NRHS = B.shape[1]
    else:
        raise ValueError("In sgetrs: B must be a vector or matrix")

    f_lapack.sgetrs_("N", &N, &NRHS, <float *>LU.data, &N,
                     <l_int *>IPIV.data, <float *>b.data, &N,
                     &info)

    assert info == 0, "Argument error in sgetrs"

    return b

def dgetrs(np.ndarray[np.float64_t, ndim=2] LU,
           np.ndarray[l_int, ndim=1] IPIV, B):
    cdef l_int N, NRHS, info
    cdef np.ndarray b

    assert_fortran_mat(LU)

    # again: workaround for 1x1-Fortran bug in NumPy < v2.0
    if (not isinstance(B, np.ndarray) or
        (B.ndim == 2 and (B.shape[0] > 1 or B.shape[1] > 1) and
         not B.flags["F_CONTIGUOUS"])):
        raise ValueError("In dgetrs: B must be a Fortran ordered NumPy array")

    b = B
    N = LU.shape[0]
    if b.ndim == 1:
        NRHS = 1
    elif b.ndim == 2:
        NRHS = b.shape[1]
    else:
        raise ValueError("In dgetrs: B must be a vector or matrix")

    f_lapack.dgetrs_("N", &N, &NRHS, <double *>LU.data, &N,
                     <l_int *>IPIV.data, <double *>b.data, &N,
                     &info)

    assert info == 0, "Argument error in dgetrs"

    return b

def cgetrs(np.ndarray[np.complex64_t, ndim=2] LU,
           np.ndarray[l_int, ndim=1] IPIV, B):
    cdef l_int N, NRHS, info
    cdef np.ndarray b

    assert_fortran_mat(LU)

    # again: workaround for 1x1-Fortran bug in NumPy < v2.0
    if (not isinstance(B, np.ndarray) or
        (B.ndim == 2 and (B.shape[0] > 1 or B.shape[1] > 1) and
         not B.flags["F_CONTIGUOUS"])):
        raise ValueError("In dgetrs: B must be a Fortran ordered NumPy array")

    b = B
    N = LU.shape[0]
    if b.ndim == 1:
        NRHS = 1
    elif b.ndim == 2:
        NRHS = b.shape[1]
    else:
        raise ValueError("In cgetrs: B must be a vector or matrix")

    f_lapack.cgetrs_("N", &N, &NRHS, <float complex *>LU.data, &N,
                     <l_int *>IPIV.data, <float complex *>b.data, &N,
                     &info)

    assert info == 0, "Argument error in cgetrs"

    return b

def zgetrs(np.ndarray[np.complex128_t, ndim=2] LU,
           np.ndarray[l_int, ndim=1] IPIV, B):
    cdef l_int N, NRHS, info
    cdef np.ndarray b

    assert_fortran_mat(LU)

    # again: workaround for 1x1-Fortran bug in NumPy < v2.0
    if (not isinstance(B, np.ndarray) or
        (B.ndim == 2 and (B.shape[0] > 1 or B.shape[1] > 1) and
         not B.flags["F_CONTIGUOUS"])):
        raise ValueError("In dgetrs: B must be a Fortran ordered NumPy array")

    b = B
    N = LU.shape[0]
    if b.ndim == 1:
        NRHS = 1
    elif b.ndim == 2:
        NRHS = b.shape[1]
    else:
        raise ValueError("In zgetrs: B must be a vector or matrix")

    f_lapack.zgetrs_("N", &N, &NRHS, <double complex *>LU.data, &N,
                     <l_int *>IPIV.data, <double complex *>b.data, &N,
                     &info)

    assert info == 0, "Argument error in zgetrs"

    return b

# Wrappers for xGECON

def sgecon(np.ndarray[np.float32_t, ndim=2] LU,
            float normA, char *norm = "1"):
    cdef l_int N, info
    cdef float rcond
    cdef np.ndarray[np.float32_t, ndim=1] work
    cdef np.ndarray[l_int, ndim=1] iwork

    assert_fortran_mat(LU)

    N = LU.shape[0]
    work = np.empty(4*N, dtype = np.float32)
    iwork = np.empty(N, dtype = f_lapack.l_int_dtype)

    f_lapack.sgecon_(norm, &N, <float *>LU.data, &N, &normA,
                     &rcond, <float *>work.data,
                     <l_int *>iwork.data, &info)

    assert info == 0, "Argument error in sgecon"

    return rcond

def dgecon(np.ndarray[np.float64_t, ndim=2] LU,
            double normA, char *norm = "1"):
    cdef l_int N, info
    cdef double rcond
    cdef np.ndarray[np.float64_t, ndim=1] work
    cdef np.ndarray[l_int, ndim=1] iwork

    assert_fortran_mat(LU)

    N = LU.shape[0]
    work = np.empty(4*N, dtype = np.float64)
    iwork = np.empty(N, dtype = f_lapack.l_int_dtype)

    f_lapack.dgecon_(norm, &N, <double *>LU.data, &N, &normA,
                     &rcond, <double *>work.data,
                     <l_int *>iwork.data, &info)

    assert info == 0, "Argument error in dgecon"

    return rcond

def cgecon(np.ndarray[np.complex64_t, ndim=2] LU,
            float normA, char *norm = "1"):
    cdef l_int N, info
    cdef float rcond
    cdef np.ndarray[np.complex64_t, ndim=1] work
    cdef np.ndarray[np.float32_t, ndim=1] rwork

    assert_fortran_mat(LU)

    N = LU.shape[0]
    work = np.empty(2*N, dtype = np.complex64)
    rwork = np.empty(2*N, dtype = np.float32)

    f_lapack.cgecon_(norm, &N, <float complex *>LU.data, &N, &normA,
                     &rcond, <float complex *>work.data,
                     <float *>rwork.data, &info)

    assert info == 0, "Argument error in cgecon"

    return rcond

def zgecon(np.ndarray[np.complex128_t, ndim=2] LU,
           double normA, char *norm = "1"):
    cdef l_int N, info
    cdef double rcond
    cdef np.ndarray[np.complex128_t, ndim=1] work
    cdef np.ndarray[np.float64_t, ndim=1] rwork

    assert_fortran_mat(LU)

    N = LU.shape[0]
    work = np.empty(2*N, dtype = np.complex128)
    rwork = np.empty(2*N, dtype = np.float64)

    f_lapack.zgecon_(norm, &N, <double complex *>LU.data, &N, &normA,
                     &rcond, <double complex *>work.data,
                     <double *>rwork.data, &info)

    assert info == 0, "Argument error in zgecon"

    return rcond

# Wrappers for xGGEV

# Helper function for xGGEV
def ggev_postprocess(dtype, alphar, alphai, vl_r=None, vr_r=None):
    # depending on whether the eigenvalues are purely real or complex,
    # some post-processing of the eigenvalues and -vectors is necessary

    indx = (alphai > 0.0).nonzero()[0]

    if indx.size:
        alpha = alphar + 1j * alphai

        if vl_r != None:
            vl = np.array(vl_r, dtype = dtype)
            for i in indx:
                vl.imag[:, i] = vl_r[:,i+1]
                vl[:, i+1] = np.conj(vl[:, i])
        else:
            vl = None

        if vr_r != None:
            vr = np.array(vr_r, dtype = dtype)
            for i in indx:
                vr.imag[:, i] = vr_r[:,i+1]
                vr[:, i+1] = np.conj(vr[:, i])
        else:
            vr = None
    else:
        alpha = alphar
        vl = vl_r
        vr = vr_r

    return (alpha, vl, vr)


def sggev(np.ndarray[np.float32_t, ndim=2] A,
          np.ndarray[np.float32_t, ndim=2] B,
          left=False, right=True):
    cdef l_int N, info, lwork
    cdef char *jobvl, *jobvr
    cdef np.ndarray[np.float32_t, ndim=2] vl_r, vr_r
    cdef float *vl_ptr, *vr_ptr, qwork
    cdef np.ndarray[np.float32_t, ndim=1] work, alphar, alphai, beta

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alphar = np.empty(N, dtype = np.float32)
    alphai = np.empty(N, dtype = np.float32)
    beta = np.empty(N, dtype = np.float32)

    if left:
        vl_r = np.empty((N,N), dtype = np.float32, order='F')
        vl_ptr = <float *>vl_r.data
        jobvl = "V"
    else:
        vl_r = None
        vl_ptr = NULL
        jobvl = "N"

    if right:
        vr_r = np.empty((N,N), dtype = np.float32, order='F')
        vr_ptr = <float *>vr_r.data
        jobvr = "V"
    else:
        vr_r = None
        vr_ptr = NULL
        jobvr = "N"

    # workspace query
    lwork = -1

    f_lapack.sggev_(jobvl, jobvr, &N, <float *>A.data, &N,
                    <float *>B.data, &N,
                    <float *>alphar.data, <float *> alphai.data,
                    <float *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    &qwork, &lwork, &info)

    assert info == 0, "Argument error in sggev"

    lwork = <l_int>qwork
    work = np.empty(lwork, dtype = np.float32)

    # Now the real calculation
    f_lapack.sggev_(jobvl, jobvr, &N, <float *>A.data, &N,
                    <float *>B.data, &N,
                    <float *>alphar.data, <float *> alphai.data,
                    <float *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    <float *>work.data, &lwork, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in sggev")

    assert info == 0, "Argument error in sggev"

    alpha, vl, vr = ggev_postprocess(np.complex64, alphar, alphai, vl_r, vr_r)

    return filter_args((True, True, left, right), (alpha, beta, vl, vr))


def dggev(np.ndarray[np.float64_t, ndim=2] A,
          np.ndarray[np.float64_t, ndim=2] B,
          left=False, right=True):
    cdef l_int N, info, lwork
    cdef char *jobvl, *jobvr
    cdef np.ndarray[np.float64_t, ndim=2] vl_r, vr_r
    cdef double *vl_ptr, *vr_ptr, qwork
    cdef np.ndarray[np.float64_t, ndim=1] work, alphar, alphai, beta

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alphar = np.empty(N, dtype = np.float64)
    alphai = np.empty(N, dtype = np.float64)
    beta = np.empty(N, dtype = np.float64)

    if left:
        vl_r = np.empty((N,N), dtype = np.float64, order='F')
        vl_ptr = <double *>vl_r.data
        jobvl = "V"
    else:
        vl_r = None
        vl_ptr = NULL
        jobvl = "N"

    if right:
        vr_r = np.empty((N,N), dtype = np.float64, order='F')
        vr_ptr = <double *>vr_r.data
        jobvr = "V"
    else:
        vr_r = None
        vr_ptr = NULL
        jobvr = "N"

    # workspace query
    lwork = -1

    f_lapack.dggev_(jobvl, jobvr, &N, <double *>A.data, &N,
                    <double *>B.data, &N,
                    <double *>alphar.data, <double *> alphai.data,
                    <double *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    &qwork, &lwork, &info)

    assert info == 0, "Argument error in dggev"

    lwork = <l_int>qwork
    work = np.empty(lwork, dtype = np.float64)

    # Now the real calculation
    f_lapack.dggev_(jobvl, jobvr, &N, <double *>A.data, &N,
                    <double *>B.data, &N,
                    <double *>alphar.data, <double *> alphai.data,
                    <double *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    <double *>work.data, &lwork, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in dggev")

    assert info == 0, "Argument error in dggev"

    alpha, vl, vr = ggev_postprocess(np.complex128, alphar, alphai, vl_r, vr_r)

    return filter_args((True, True, left, right), (alpha, beta, vl, vr))


def cggev(np.ndarray[np.complex64_t, ndim=2] A,
          np.ndarray[np.complex64_t, ndim=2] B,
          left=False, right=True):
    cdef l_int N, info, lwork
    cdef char *jobvl, *jobvr
    cdef np.ndarray[np.complex64_t, ndim=2] vl, vr
    cdef float complex *vl_ptr, *vr_ptr, qwork
    cdef np.ndarray[np.complex64_t, ndim=1] work, alpha, beta
    cdef np.ndarray[np.float32_t, ndim=1] rwork

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alpha = np.empty(N, dtype = np.complex64)
    beta = np.empty(N, dtype = np.complex64)

    if left:
        vl = np.empty((N,N), dtype = np.complex64, order='F')
        vl_ptr = <float complex *>vl.data
        jobvl = "V"
    else:
        vl_ptr = NULL
        jobvl = "N"

    if right:
        vr = np.empty((N,N), dtype = np.complex64, order='F')
        vr_ptr = <float complex *>vr.data
        jobvr = "V"
    else:
        vr_ptr = NULL
        jobvr = "N"

    rwork = np.empty(8 * N, dtype = np.float32)

    # workspace query
    lwork = -1
    work = np.empty(1, dtype = np.complex64)

    f_lapack.cggev_(jobvl, jobvr, &N, <float complex *>A.data, &N,
                    <float complex *>B.data, &N,
                    <float complex *>alpha.data, <float complex *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    &qwork, &lwork,
                    <float *>rwork.data, &info)

    assert info == 0, "Argument error in cggev"

    lwork = <l_int>qwork.real

    work = np.empty(lwork, dtype = np.complex64)

    # Now the real calculation
    f_lapack.cggev_(jobvl, jobvr, &N, <float complex *>A.data, &N,
                    <float complex *>B.data, &N,
                    <float complex *>alpha.data, <float complex *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    <float complex *>work.data, &lwork,
                    <float *>rwork.data, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in cggev")

    assert info == 0, "Argument error in cggev"

    return filter_args((True, True, left, right), (alpha, beta, vl, vr))


def zggev(np.ndarray[np.complex128_t, ndim=2] A,
          np.ndarray[np.complex128_t, ndim=2] B,
          left=False, right=True):
    cdef l_int N, info, lwork
    cdef char *jobvl, *jobvr
    cdef np.ndarray[np.complex128_t, ndim=2] vl, vr
    cdef double complex *vl_ptr, *vr_ptr, qwork
    cdef np.ndarray[np.complex128_t, ndim=1] work, alpha, beta
    cdef np.ndarray[np.float64_t, ndim=1] rwork

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alpha = np.empty(N, dtype = np.complex128)
    beta = np.empty(N, dtype = np.complex128)

    if left:
        vl = np.empty((N,N), dtype = np.complex128, order='F')
        vl_ptr = <double complex *>vl.data
        jobvl = "V"
    else:
        vl_ptr = NULL
        jobvl = "N"

    if right:
        vr = np.empty((N,N), dtype = np.complex128, order='F')
        vr_ptr = <double complex *>vr.data
        jobvr = "V"
    else:
        vr_ptr = NULL
        jobvr = "N"

    rwork = np.empty(8 * N, dtype = np.float64)

    # workspace query
    lwork = -1
    work = np.empty(1, dtype = np.complex128)

    f_lapack.zggev_(jobvl, jobvr, &N, <double complex *>A.data, &N,
                    <double complex *>B.data, &N,
                    <double complex *>alpha.data, <double complex *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    &qwork, &lwork,
                    <double *>rwork.data, &info)

    assert info == 0, "Argument error in zggev"

    lwork = <l_int>qwork.real
    work = np.empty(lwork, dtype = np.complex128)

    # Now the real calculation
    f_lapack.zggev_(jobvl, jobvr, &N, <double complex *>A.data, &N,
                    <double complex *>B.data, &N,
                    <double complex *>alpha.data, <double complex *>beta.data,
                    vl_ptr, &N, vr_ptr, &N,
                    <double complex *>work.data, &lwork,
                    <double *>rwork.data, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in zggev")

    assert info == 0, "Argument error in zggev"

    return filter_args((True, True, left, right), (alpha, beta, vl, vr))


# Wrapper for xGEES
def sgees(np.ndarray[np.float32_t, ndim=2] A,
          calc_q=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvs
    cdef float *vs_ptr, qwork
    cdef np.ndarray[np.float32_t, ndim=2] vs
    cdef np.ndarray[np.float32_t] wr, wi, work

    assert_fortran_mat(A)

    N = A.shape[0]
    wr = np.empty(N, dtype = np.float32)
    wi = np.empty(N, dtype = np.float32)

    if calc_q:
        vs = np.empty((N,N), dtype = np.float32, order='F')
        vs_ptr = <float *>vs.data
        jobvs = "V"
    else:
        vs_ptr = NULL
        jobvs = "N"

    # workspace query
    lwork = -1
    f_lapack.sgees_(jobvs, "N", NULL, &N, <float *>A.data, &N,
                    &sdim, <float *>wr.data, <float *>wi.data, vs_ptr, &N,
                    &qwork, &lwork, NULL, &info)

    assert info == 0, "Argument error in sgees"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float32)

    # Now the real calculation
    f_lapack.sgees_(jobvs, "N", NULL, &N, <float *>A.data, &N,
                    &sdim, <float *>wr.data, <float *>wi.data, vs_ptr, &N,
                    <float *>work.data, &lwork, NULL, &info)

    if info > 0:
        raise LinAlgError("QR iteration failed to converge in sgees")

    assert info == 0, "Argument error in sgees"

    if wi.nonzero()[0].size:
        w = wr + 1j * wi
    else:
        w = wr

    return filter_args((True, calc_q, calc_ev), (A, vs, w))


def dgees(np.ndarray[np.float64_t, ndim=2] A,
          calc_q=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvs
    cdef double *vs_ptr, qwork
    cdef np.ndarray[np.float64_t, ndim=2] vs
    cdef np.ndarray[np.float64_t] wr, wi, work

    assert_fortran_mat(A)

    N = A.shape[0]
    wr = np.empty(N, dtype = np.float64)
    wi = np.empty(N, dtype = np.float64)

    if calc_q:
        vs = np.empty((N,N), dtype = np.float64, order='F')
        vs_ptr = <double *>vs.data
        jobvs = "V"
    else:
        vs_ptr = NULL
        jobvs = "N"

    # workspace query
    lwork = -1
    f_lapack.dgees_(jobvs, "N", NULL, &N, <double *>A.data, &N,
                    &sdim, <double *>wr.data, <double *>wi.data, vs_ptr, &N,
                    &qwork, &lwork, NULL, &info)

    assert info == 0, "Argument error in dgees"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float64)

    # Now the real calculation
    f_lapack.dgees_(jobvs, "N", NULL, &N, <double *>A.data, &N,
                    &sdim, <double *>wr.data, <double *>wi.data, vs_ptr, &N,
                    <double *>work.data, &lwork, NULL, &info)

    if info > 0:
        raise LinAlgError("QR iteration failed to converge in dgees")

    assert info == 0, "Argument error in dgees"

    if wi.nonzero()[0].size:
        w = wr + 1j * wi
    else:
        w = wr

    return filter_args((True, calc_q, calc_ev), (A, vs, w))


def cgees(np.ndarray[np.complex64_t, ndim=2] A,
          calc_q=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvs
    cdef float complex *vs_ptr, qwork
    cdef np.ndarray[np.complex64_t, ndim=2] vs
    cdef np.ndarray[np.complex64_t] w, work
    cdef np.ndarray[np.float32_t] rwork

    assert_fortran_mat(A)

    N = A.shape[0]
    w = np.empty(N, dtype = np.complex64)
    rwork = np.empty(N, dtype = np.float32)

    if calc_q:
        vs = np.empty((N,N), dtype = np.complex64, order='F')
        vs_ptr = <float complex *>vs.data
        jobvs = "V"
    else:
        vs_ptr = NULL
        jobvs = "N"

    # workspace query
    lwork = -1
    f_lapack.cgees_(jobvs, "N", NULL, &N, <float complex *>A.data, &N,
                    &sdim, <float complex *>w.data, vs_ptr, &N,
                    &qwork, &lwork, <float *>rwork.data, NULL, &info)

    assert info == 0, "Argument error in cgees"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex64)

    # Now the real calculation
    f_lapack.cgees_(jobvs, "N", NULL, &N, <float complex *>A.data, &N,
                    &sdim, <float complex *>w.data, vs_ptr, &N,
                    <float complex *>work.data, &lwork,
                    <float *>rwork.data, NULL, &info)

    if info > 0:
        raise LinAlgError("QR iteration failed to converge in cgees")

    assert info == 0, "Argument error in cgees"

    return filter_args((True, calc_q, calc_ev), (A, vs, w))


def zgees(np.ndarray[np.complex128_t, ndim=2] A,
          calc_q=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvs
    cdef double complex *vs_ptr, qwork
    cdef np.ndarray[np.complex128_t, ndim=2] vs
    cdef np.ndarray[np.complex128_t] w, work
    cdef np.ndarray[np.float64_t] rwork

    assert_fortran_mat(A)

    N = A.shape[0]
    w = np.empty(N, dtype = np.complex128)
    rwork = np.empty(N, dtype = np.float64)

    if calc_q:
        vs = np.empty((N,N), dtype = np.complex128, order='F')
        vs_ptr = <double complex *>vs.data
        jobvs = "V"
    else:
        vs_ptr = NULL
        jobvs = "N"

    # workspace query
    lwork = -1
    f_lapack.zgees_(jobvs, "N", NULL, &N, <double complex *>A.data, &N,
                    &sdim, <double complex *>w.data, vs_ptr, &N,
                    &qwork, &lwork, <double *>rwork.data, NULL, &info)

    assert info == 0, "Argument error in zgees"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex128)

    # Now the real calculation
    f_lapack.zgees_(jobvs, "N", NULL, &N, <double complex *>A.data, &N,
                    &sdim, <double complex *>w.data, vs_ptr, &N,
                    <double complex *>work.data, &lwork,
                    <double *>rwork.data, NULL, &info)

    if info > 0:
        raise LinAlgError("QR iteration failed to converge in zgees")

    assert info == 0, "Argument error in zgees"

    return filter_args((True, calc_q, calc_ev), (A, vs, w))

# Wrapper for xTRSEN
def strsen(np.ndarray[l_logical] select,
           np.ndarray[np.float32_t, ndim=2] T,
           np.ndarray[np.float32_t, ndim=2] Q=None,
           calc_ev=True):
    cdef l_int N, M, lwork, liwork, qiwork, info
    cdef char *compq
    cdef float qwork, *q_ptr
    cdef np.ndarray[np.float32_t] wr, wi, work
    cdef np.ndarray[l_int] iwork

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    wr = np.empty(N, dtype = np.float32)
    wi = np.empty(N, dtype = np.float32)

    if Q is not None:
        compq = "V"
        q_ptr = <float *>Q.data
    else:
        compq = "N"
        q_ptr = NULL

    # workspace query
    lwork = liwork = -1
    f_lapack.strsen_("N", compq, <l_logical *>select.data,
                     &N, <float *>T.data, &N, q_ptr, &N,
                     <float *>wr.data, <float *>wi.data, &M, NULL, NULL,
                     &qwork, &lwork, &qiwork, &liwork, &info)

    assert info == 0, "Argument error in strsen"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float32)
    liwork = qiwork
    iwork = np.empty(liwork, dtype = f_lapack.l_int_dtype)

    # Now the real calculation
    f_lapack.strsen_("N", compq, <l_logical *>select.data,
                     &N, <float *>T.data, &N, q_ptr, &N,
                     <float *>wr.data, <float *>wi.data, &M, NULL, NULL,
                     <float *>work.data, &lwork,
                     <int *>iwork.data, &liwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in strsen"

    if wi.nonzero()[0].size:
        w = wr + 1j * wi
    else:
        w = wr

    return filter_args((True, Q is not None, calc_ev), (T, Q, w))


def dtrsen(np.ndarray[l_logical] select,
           np.ndarray[np.float64_t, ndim=2] T,
           np.ndarray[np.float64_t, ndim=2] Q=None,
           calc_ev=True):
    cdef l_int N, M, lwork, liwork, qiwork, info
    cdef char *compq
    cdef double qwork, *q_ptr
    cdef np.ndarray[np.float64_t] wr, wi, work
    cdef np.ndarray[l_int] iwork

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    wr = np.empty(N, dtype = np.float64)
    wi = np.empty(N, dtype = np.float64)

    if Q is not None:
        compq = "V"
        q_ptr = <double *>Q.data
    else:
        compq = "N"
        q_ptr = NULL

    # workspace query
    lwork = liwork = -1
    f_lapack.dtrsen_("N", compq, <l_logical *>select.data,
                     &N, <double *>T.data, &N, q_ptr, &N,
                     <double *>wr.data, <double *>wi.data, &M, NULL, NULL,
                     &qwork, &lwork, &qiwork, &liwork, &info)

    assert info == 0, "Argument error in dtrsen"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float64)
    liwork = qiwork
    iwork = np.empty(liwork, dtype = f_lapack.l_int_dtype)

    # Now the real calculation
    f_lapack.dtrsen_("N", compq, <l_logical *>select.data,
                     &N, <double *>T.data, &N, q_ptr, &N,
                     <double *>wr.data, <double *>wi.data, &M, NULL, NULL,
                     <double *>work.data, &lwork,
                     <int *>iwork.data, &liwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in dtrsen"

    if wi.nonzero()[0].size:
        w = wr + 1j * wi
    else:
        w = wr

    return filter_args((True, Q is not None, calc_ev), (T, Q, w))


def ctrsen(np.ndarray[l_logical] select,
           np.ndarray[np.complex64_t, ndim=2] T,
           np.ndarray[np.complex64_t, ndim=2] Q=None,
           calc_ev=True):
    cdef l_int N, M, lwork, info
    cdef char *compq
    cdef float complex qwork, *q_ptr
    cdef np.ndarray[np.complex64_t] w, work

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    w = np.empty(N, dtype = np.complex64)

    if Q is not None:
        compq = "V"
        q_ptr = <float complex *>Q.data
    else:
        compq = "N"
        q_ptr = NULL

    # workspace query
    lwork = -1
    f_lapack.ctrsen_("N", compq, <l_logical *>select.data,
                     &N, <float complex *>T.data, &N, q_ptr, &N,
                     <float complex *>w.data, &M, NULL, NULL,
                     &qwork, &lwork, &info)

    assert info == 0, "Argument error in ctrsen"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex64)

    # Now the real calculation
    f_lapack.ctrsen_("N", compq, <l_logical *>select.data,
                     &N, <float complex *>T.data, &N, q_ptr, &N,
                     <float complex *>w.data, &M, NULL, NULL,
                     <float complex *>work.data, &lwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in ctrsen"

    return filter_args((True, Q is not None, calc_ev), (T, Q, w))


def ztrsen(np.ndarray[l_logical] select,
           np.ndarray[np.complex128_t, ndim=2] T,
           np.ndarray[np.complex128_t, ndim=2] Q=None,
           calc_ev=True):
    cdef l_int N, MM, M, lwork, info
    cdef char *compq
    cdef double complex qwork, *q_ptr
    cdef np.ndarray[np.complex128_t] w, work

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    w = np.empty(N, dtype = np.complex128)

    if Q is not None:
        compq = "V"
        q_ptr = <double complex *>Q.data
    else:
        compq = "N"
        q_ptr = NULL

    # workspace query
    lwork = -1
    f_lapack.ztrsen_("N", compq, <l_logical *>select.data,
                     &N, <double complex *>T.data, &N, q_ptr, &N,
                     <double complex *>w.data, &M, NULL, NULL,
                     &qwork, &lwork, &info)

    assert info == 0, "Argument error in ztrsen"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex128)

    # Now the real calculation
    f_lapack.ztrsen_("N", compq, <l_logical *>select.data,
                     &N, <double complex *>T.data, &N, q_ptr, &N,
                     <double complex *>w.data, &M, NULL, NULL,
                     <double complex *>work.data, &lwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in ztrsen"

    return filter_args((True, Q is not None, calc_ev), (T, Q, w))


# Helper function for xTREVC and xTGEVC
def txevc_postprocess(dtype, T, vreal, np.ndarray[l_logical] select):
    cdef int N, M, i, m, indx

    N = T.shape[0]
    if select is None:
        select = np.ones(N, dtype = f_lapack.l_logical_dtype)
    selindx = select.nonzero()[0]
    M = selindx.size

    v = np.empty((N, M), dtype = dtype, order='F')

    indx = 0
    for m in xrange(M):
        k = selindx[m]

        if k < N-1 and T[k+1,k]:
            # we have the situation of a 2x2 block, and
            # the eigenvalue with the positive imaginary part desired
            v[:, m] = vreal[:, indx] + 1j * vreal[:, indx + 1]

            # Check if the eigenvalue with negative real part is also
            # selected, if it is, we need the same entries in vr
            if not select[k+1]:
                indx += 2
        elif k > 0 and T[k,k-1]:
            # we have the situation of a 2x2 block, and
            # the eigenvalue with the negative imaginary part desired
            v[:, m] = vreal[:, indx] - 1j * vreal[:, indx + 1]

            indx += 2
        else:
            # real eigenvalue
            v[:, m] = vreal[:, indx]

            indx += 1
    return v


# Wrappers for xTREVC
def strevc(np.ndarray[np.float32_t, ndim=2] T,
           np.ndarray[np.float32_t, ndim=2] Q=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.float32_t, ndim=2] vl_r, vr_r
    cdef float *vl_r_ptr, *vr_r_ptr
    cdef np.ndarray[l_logical] select_cpy
    cdef l_logical *select_ptr
    cdef np.ndarray[np.float32_t] work

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    work = np.empty(4*N, dtype = np.float32)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        # Correct for possible additional storage if a single complex
        # eigenvalue is selected.
        # For that: Figure out the positions of the 2x2 blocks.
        cmplxindx = np.diagonal(T, -1).nonzero()[0]
        for i in cmplxindx:
            if bool(select[i]) != bool(select[i+1]):
                MM += 1

        # Select is overwritten in strevc.
        select_cpy = np.array(select, dtype = f_lapack.l_logical_dtype,
                              order = 'F')
        select_ptr = <l_logical *>select_cpy.data
    else:
        MM = N
        select_ptr = NULL
        if Q is not None:
            howmny = "B"
        else:
            howmny = "A"

    if left:
        if Q is not None and select is None:
            vl_r = np.asfortranarray(Q.copy())
        else:
            vl_r = np.empty((N, MM), dtype = np.float32, order='F')
        vl_r_ptr = <float *>vl_r.data
    else:
        vl_r_ptr = NULL

    if right:
        if Q is not None and select is None:
            vr_r = np.asfortranarray(Q.copy())
        else:
            vr_r = np.empty((N, MM), dtype = np.float32, order='F')
        vr_r_ptr = <float *>vr_r.data
    else:
        vr_r_ptr = NULL

    f_lapack.strevc_(side, howmny, select_ptr,
                     &N, <float *>T.data, &N,
                     vl_r_ptr, &N, vr_r_ptr, &N, &MM, &M,
                     <float *>work.data, &info)

    assert info == 0, "Argument error in strevc"
    assert MM == M, "Unexpected number of eigenvectors returned in strevc"

    if select is not None and Q is not None:
        if left:
            vl_r = np.asfortranarray(np.dot(Q, vl_r))
        if right:
            vr_r = np.asfortranarray(np.dot(Q, vr_r))

    # If there are complex eigenvalues, we need to postprocess the
    # eigenvectors.
    if np.diagonal(T, -1).nonzero()[0].size:
        if left:
            vl = txevc_postprocess(np.complex64, T, vl_r, select)
        if right:
            vr = txevc_postprocess(np.complex64, T, vr_r, select)
    else:
        if left:
            vl = vl_r
        if right:
            vr = vr_r

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


def dtrevc(np.ndarray[np.float64_t, ndim=2] T,
           np.ndarray[np.float64_t, ndim=2] Q=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.float64_t, ndim=2] vl_r, vr_r
    cdef double *vl_r_ptr, *vr_r_ptr
    cdef np.ndarray[l_logical] select_cpy
    cdef l_logical *select_ptr
    cdef np.ndarray[np.float64_t] work

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    work = np.empty(4*N, dtype = np.float64)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        # Correct for possible additional storage if a single complex
        # eigenvalue is selected.
        # For that: Figure out the positions of the 2x2 blocks.
        cmplxindx = np.diagonal(T, -1).nonzero()[0]
        for i in cmplxindx:
            if bool(select[i]) != bool(select[i+1]):
                MM += 1

        # Select is overwritten in dtrevc.
        select_cpy = np.array(select, dtype = f_lapack.l_logical_dtype,
                              order = 'F')
        select_ptr = <l_logical *>select_cpy.data
    else:
        MM = N
        select_ptr = NULL
        if Q is not None:
            howmny = "B"
        else:
            howmny = "A"

    if left:
        if Q is not None and select is None:
            vl_r = np.asfortranarray(Q.copy())
        else:
            vl_r = np.empty((N, MM), dtype = np.float64, order='F')
        vl_r_ptr = <double *>vl_r.data
    else:
        vl_r_ptr = NULL

    if right:
        if Q is not None and select is None:
            vr_r = np.asfortranarray(Q.copy())
        else:
            vr_r = np.empty((N, MM), dtype = np.float64, order='F')
        vr_r_ptr = <double *>vr_r.data
    else:
        vr_r_ptr = NULL

    f_lapack.dtrevc_(side, howmny, select_ptr,
                     &N, <double *>T.data, &N,
                     vl_r_ptr, &N, vr_r_ptr, &N, &MM, &M,
                     <double *>work.data, &info)

    assert info == 0, "Argument error in dtrevc"
    assert MM == M, "Unexpected number of eigenvectors returned in dtrevc"

    if select is not None and Q is not None:
        if left:
            vl_r = np.asfortranarray(np.dot(Q, vl_r))
        if right:
            vr_r = np.asfortranarray(np.dot(Q, vr_r))

    # If there are complex eigenvalues, we need to postprocess the eigenvectors
    if np.diagonal(T, -1).nonzero()[0].size:
        if left:
            vl = txevc_postprocess(np.complex128, T, vl_r, select)
        if right:
            vr = txevc_postprocess(np.complex128, T, vr_r, select)
    else:
        if left:
            vl = vl_r
        if right:
            vr = vr_r

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


def ctrevc(np.ndarray[np.complex64_t, ndim=2] T,
           np.ndarray[np.complex64_t, ndim=2] Q=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.complex64_t, ndim=2] vl, vr
    cdef float complex *vl_ptr, *vr_ptr
    cdef l_logical *select_ptr
    cdef np.ndarray[np.complex64_t] work
    cdef np.ndarray[np.float32_t] rwork

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    work = np.empty(2*N, dtype = np.complex64)
    rwork = np.empty(N, dtype = np.float32)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        select_ptr = <l_logical *>select.data
    else:
        MM = N
        select_ptr = NULL
        if Q is not None:
            howmny = "B"
        else:
            howmny = "A"

    if left:
        if Q is not None and select is None:
            vl = np.asfortranarray(Q.copy())
        else:
            vl = np.empty((N, MM), dtype = np.complex64, order='F')
        vl_ptr = <float complex *>vl.data
    else:
        vl_ptr = NULL

    if right:
        if Q is not None and select is None:
            vr = np.asfortranarray(Q.copy())
        else:
            vr = np.empty((N, MM), dtype = np.complex64, order='F')
        vr_ptr = <float complex *>vr.data
    else:
        vr_ptr = NULL

    f_lapack.ctrevc_(side, howmny, select_ptr,
                     &N, <float complex *>T.data, &N,
                     vl_ptr, &N, vr_ptr, &N, &MM, &M,
                     <float complex *>work.data, <float *>rwork.data, &info)

    assert info == 0, "Argument error in ctrevc"
    assert MM == M, "Unexpected number of eigenvectors returned in ctrevc"

    if select is not None and Q is not None:
        if left:
            vl = np.asfortranarray(np.dot(Q, vl))
        if right:
            vr = np.asfortranarray(np.dot(Q, vr))

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


def ztrevc(np.ndarray[np.complex128_t, ndim=2] T,
           np.ndarray[np.complex128_t, ndim=2] Q=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.complex128_t, ndim=2] vl, vr
    cdef double complex *vl_ptr, *vr_ptr
    cdef l_logical *select_ptr
    cdef np.ndarray[np.complex128_t] work
    cdef np.ndarray[np.float64_t] rwork

    assert_fortran_mat(T, Q)

    N = T.shape[0]
    work = np.empty(2*N, dtype = np.complex128)
    rwork = np.empty(N, dtype = np.float64)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        select_ptr = <l_logical *>select.data
    else:
        MM = N
        select_ptr = NULL
        if Q is not None:
            howmny = "B"
        else:
            howmny = "A"

    if left:
        if Q is not None and select is None:
            vl = np.asfortranarray(Q.copy())
        else:
            vl = np.empty((N, MM), dtype = np.complex128, order='F')
        vl_ptr = <double complex *>vl.data
    else:
        vl_ptr = NULL

    if right:
        if Q is not None and select is None:
            vr = np.asfortranarray(Q.copy())
        else:
            vr = np.empty((N, MM), dtype = np.complex128, order='F')
        vr_ptr = <double complex *>vr.data
    else:
        vr_ptr = NULL

    f_lapack.ztrevc_(side, howmny, select_ptr,
                     &N, <double complex *>T.data, &N,
                     vl_ptr, &N, vr_ptr, &N, &MM, &M,
                     <double complex *>work.data, <double *>rwork.data, &info)

    assert info == 0, "Argument error in ztrevc"
    assert MM == M, "Unexpected number of eigenvectors returned in ztrevc"

    if select is not None and Q is not None:
        if left:
            vl = np.asfortranarray(np.dot(Q, vl))
        if right:
            vr = np.asfortranarray(np.dot(Q, vr))

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


# wrappers for xGGES
def sgges(np.ndarray[np.float32_t, ndim=2] A,
          np.ndarray[np.float32_t, ndim=2] B,
          calc_q=True, calc_z=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvsl, *jobvsr
    cdef float *vsl_ptr, *vsr_ptr, qwork
    cdef np.ndarray[np.float32_t, ndim=2] vsl, vsr
    cdef np.ndarray[np.float32_t] alphar, alphai, beta, work

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alphar = np.empty(N, dtype = np.float32)
    alphai = np.empty(N, dtype = np.float32)
    beta = np.empty(N, dtype = np.float32)

    if calc_q:
        vsl = np.empty((N,N), dtype = np.float32, order='F')
        vsl_ptr = <float *>vsl.data
        jobvsl = "V"
    else:
        vsl = None
        vsl_ptr = NULL
        jobvsl = "N"

    if calc_z:
        vsr = np.empty((N,N), dtype = np.float32, order='F')
        vsr_ptr = <float *>vsr.data
        jobvsr = "V"
    else:
        vsr = None
        vsr_ptr = NULL
        jobvsr = "N"

    # workspace query
    lwork = -1
    f_lapack.sgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <float *>A.data, &N,
                    <float *>B.data, &N, &sdim,
                    <float *>alphar.data, <float *>alphai.data,
                    <float *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    &qwork, &lwork, NULL, &info)

    assert info == 0, "Argument error in zgees"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float32)

    # Now the real calculation
    f_lapack.sgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <float *>A.data, &N,
                    <float *>B.data, &N, &sdim,
                    <float *>alphar.data, <float *>alphai.data,
                    <float *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    <float *>work.data, &lwork, NULL, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in sgges")

    assert info == 0, "Argument error in zgees"

    if alphai.nonzero()[0].size:
        alpha = alphar + 1j * alphai
    else:
        alpha = alphar

    return filter_args((True, True, calc_q, calc_z, calc_ev, calc_ev),
                       (A, B, vsl, vsr, alpha, beta))


def dgges(np.ndarray[np.float64_t, ndim=2] A,
          np.ndarray[np.float64_t, ndim=2] B,
          calc_q=True, calc_z=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvsl, *jobvsr
    cdef double *vsl_ptr, *vsr_ptr, qwork
    cdef np.ndarray[np.float64_t, ndim=2] vsl, vsr
    cdef np.ndarray[np.float64_t] alphar, alphai, beta, work

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alphar = np.empty(N, dtype = np.float64)
    alphai = np.empty(N, dtype = np.float64)
    beta = np.empty(N, dtype = np.float64)

    if calc_q:
        vsl = np.empty((N,N), dtype = np.float64, order='F')
        vsl_ptr = <double *>vsl.data
        jobvsl = "V"
    else:
        vsl = None
        vsl_ptr = NULL
        jobvsl = "N"

    if calc_z:
        vsr = np.empty((N,N), dtype = np.float64, order='F')
        vsr_ptr = <double *>vsr.data
        jobvsr = "V"
    else:
        vsr = None
        vsr_ptr = NULL
        jobvsr = "N"

    # workspace query
    lwork = -1
    f_lapack.dgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <double *>A.data, &N,
                    <double *>B.data, &N, &sdim,
                    <double *>alphar.data, <double *>alphai.data,
                    <double *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    &qwork, &lwork, NULL, &info)

    assert info == 0, "Argument error in zgees"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float64)

    # Now the real calculation
    f_lapack.dgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <double *>A.data, &N,
                    <double *>B.data, &N, &sdim,
                    <double *>alphar.data, <double *>alphai.data,
                    <double *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    <double *>work.data, &lwork, NULL, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in dgges")

    assert info == 0, "Argument error in zgees"

    if alphai.nonzero()[0].size:
        alpha = alphar + 1j * alphai
    else:
        alpha = alphar

    return filter_args((True, True, calc_q, calc_z, calc_ev, calc_ev),
                       (A, B, vsl, vsr, alpha, beta))


def cgges(np.ndarray[np.complex64_t, ndim=2] A,
          np.ndarray[np.complex64_t, ndim=2] B,
          calc_q=True, calc_z=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvsl, *jobvsr
    cdef float complex *vsl_ptr, *vsr_ptr, qwork
    cdef np.ndarray[np.complex64_t, ndim=2] vsl, vsr
    cdef np.ndarray[np.complex64_t] alpha, beta, work
    cdef np.ndarray[np.float32_t] rwork

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alpha = np.empty(N, dtype = np.complex64)
    beta = np.empty(N, dtype = np.complex64)
    rwork = np.empty(8*N, dtype = np.float32)

    if calc_q:
        vsl = np.empty((N,N), dtype = np.complex64, order='F')
        vsl_ptr = <float complex *>vsl.data
        jobvsl = "V"
    else:
        vsl = None
        vsl_ptr = NULL
        jobvsl = "N"

    if calc_z:
        vsr = np.empty((N,N), dtype = np.complex64, order='F')
        vsr_ptr = <float complex *>vsr.data
        jobvsr = "V"
    else:
        vsr = None
        vsr_ptr = NULL
        jobvsr = "N"

    # workspace query
    lwork = -1
    f_lapack.cgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <float complex *>A.data, &N,
                    <float complex *>B.data, &N, &sdim,
                    <float complex *>alpha.data, <float complex *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    &qwork, &lwork, <float *>rwork.data, NULL, &info)

    assert info == 0, "Argument error in zgees"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex64)

    # Now the real calculation
    f_lapack.cgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <float complex *>A.data, &N,
                    <float complex *>B.data, &N, &sdim,
                    <float complex *>alpha.data, <float complex *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    <float complex *>work.data, &lwork,
                    <float *>rwork.data, NULL, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in cgges")

    assert info == 0, "Argument error in zgees"

    return filter_args((True, True, calc_q, calc_z, calc_ev, calc_ev),
                       (A, B, vsl, vsr, alpha, beta))


def zgges(np.ndarray[np.complex128_t, ndim=2] A,
          np.ndarray[np.complex128_t, ndim=2] B,
          calc_q=True, calc_z=True, calc_ev=True):
    cdef l_int N, lwork, sdim, info
    cdef char *jobvsl, *jobvsr
    cdef double complex *vsl_ptr, *vsr_ptr, qwork
    cdef np.ndarray[np.complex128_t, ndim=2] vsl, vsr
    cdef np.ndarray[np.complex128_t] alpha, beta, work
    cdef np.ndarray[np.float64_t] rwork

    assert_fortran_mat(A, B)

    N = A.shape[0]
    alpha = np.empty(N, dtype = np.complex128)
    beta = np.empty(N, dtype = np.complex128)
    rwork = np.empty(8*N, dtype = np.float64)

    if calc_q:
        vsl = np.empty((N,N), dtype = np.complex128, order='F')
        vsl_ptr = <double complex *>vsl.data
        jobvsl = "V"
    else:
        vsl = None
        vsl_ptr = NULL
        jobvsl = "N"

    if calc_z:
        vsr = np.empty((N,N), dtype = np.complex128, order='F')
        vsr_ptr = <double complex *>vsr.data
        jobvsr = "V"
    else:
        vsr = None
        vsr_ptr = NULL
        jobvsr = "N"

    # workspace query
    lwork = -1
    f_lapack.zgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <double complex *>A.data, &N,
                    <double complex *>B.data, &N, &sdim,
                    <double complex *>alpha.data, <double complex *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    &qwork, &lwork, <double *>rwork.data, NULL, &info)

    assert info == 0, "Argument error in zgees"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex128)

    # Now the real calculation
    f_lapack.zgges_(jobvsl, jobvsr, "N", NULL,
                    &N, <double complex *>A.data, &N,
                    <double complex *>B.data, &N, &sdim,
                    <double complex *>alpha.data, <double complex *>beta.data,
                    vsl_ptr, &N, vsr_ptr, &N,
                    <double complex *>work.data, &lwork,
                    <double *>rwork.data, NULL, &info)

    if info > 0:
        raise LinAlgError("QZ iteration failed to converge in zgges")

    assert info == 0, "Argument error in zgees"

    return filter_args((True, True, calc_q, calc_z, calc_ev, calc_ev),
                       (A, B, vsl, vsr, alpha, beta))


# wrappers for xTGSEN
def stgsen(np.ndarray[l_logical] select,
           np.ndarray[np.float32_t, ndim=2] S,
           np.ndarray[np.float32_t, ndim=2] T,
           np.ndarray[np.float32_t, ndim=2] Q=None,
           np.ndarray[np.float32_t, ndim=2] Z=None,
           calc_ev=True):
    cdef l_int N, M, lwork, liwork, qiwork, info, ijob
    cdef l_logical wantq, wantz
    cdef float qwork, *q_ptr, *z_ptr
    cdef np.ndarray[np.float32_t] alphar, alphai, beta, work
    cdef np.ndarray[l_int] iwork

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    alphar = np.empty(N, dtype = np.float32)
    alphai = np.empty(N, dtype = np.float32)
    beta = np.empty(N, dtype = np.float32)
    ijob = 0

    if Q is not None:
        wantq = 1
        q_ptr = <float *>Q.data
    else:
        wantq = 0
        q_ptr = NULL

    if Z is not None:
        wantz = 1
        z_ptr = <float *>Z.data
    else:
        wantz = 0
        z_ptr = NULL

    # workspace query
    lwork = -1
    liwork = -1
    f_lapack.stgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <float *>S.data, &N,
                     <float *>T.data, &N,
                     <float *>alphar.data, <float *>alphai.data,
                     <float *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     &qwork, &lwork, &qiwork, &liwork, &info)

    assert info == 0, "Argument error in stgsen"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float32)
    liwork = qiwork
    iwork = np.empty(liwork, dtype = int_dtype)

    # Now the real calculation
    f_lapack.stgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <float *>S.data, &N,
                     <float *>T.data, &N,
                     <float *>alphar.data, <float *>alphai.data,
                     <float *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     <float *>work.data, &lwork,
                     <l_int *>iwork.data, &liwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in stgsen"

    if alphai.nonzero()[0].size:
        alpha = alphar + 1j * alphai
    else:
        alpha = alphar

    return filter_args((True, True, Q is not None, Z is not None,
                        calc_ev, calc_ev),
                       (S, T, Q, Z, alpha, beta))


def dtgsen(np.ndarray[l_logical] select,
           np.ndarray[np.float64_t, ndim=2] S,
           np.ndarray[np.float64_t, ndim=2] T,
           np.ndarray[np.float64_t, ndim=2] Q=None,
           np.ndarray[np.float64_t, ndim=2] Z=None,
           calc_ev=True):
    cdef l_int N, M, lwork, liwork, qiwork, info, ijob
    cdef l_logical wantq, wantz
    cdef double qwork, *q_ptr, *z_ptr
    cdef np.ndarray[np.float64_t] alphar, alphai, beta, work
    cdef np.ndarray[l_int] iwork

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    alphar = np.empty(N, dtype = np.float64)
    alphai = np.empty(N, dtype = np.float64)
    beta = np.empty(N, dtype = np.float64)
    ijob = 0

    if Q is not None:
        wantq = 1
        q_ptr = <double *>Q.data
    else:
        wantq = 0
        q_ptr = NULL

    if Z is not None:
        wantz = 1
        z_ptr = <double *>Z.data
    else:
        wantz = 0
        z_ptr = NULL

    # workspace query
    lwork = -1
    liwork = -1
    f_lapack.dtgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <double *>S.data, &N,
                     <double *>T.data, &N,
                     <double *>alphar.data, <double *>alphai.data,
                     <double *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     &qwork, &lwork, &qiwork, &liwork, &info)

    assert info == 0, "Argument error in dtgsen"

    lwork = <int>qwork
    work = np.empty(lwork, dtype = np.float64)
    liwork = qiwork
    iwork = np.empty(liwork, dtype = int_dtype)

    # Now the real calculation
    f_lapack.dtgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <double *>S.data, &N,
                     <double *>T.data, &N,
                     <double *>alphar.data, <double *>alphai.data,
                     <double *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     <double *>work.data, &lwork,
                     <l_int *>iwork.data, &liwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in dtgsen"

    if alphai.nonzero()[0].size:
        alpha = alphar + 1j * alphai
    else:
        alpha = alphar

    return filter_args((True, True, Q is not None, Z is not None,
                        calc_ev, calc_ev),
                       (S, T, Q, Z, alpha, beta))


def ctgsen(np.ndarray[l_logical] select,
           np.ndarray[np.complex64_t, ndim=2] S,
           np.ndarray[np.complex64_t, ndim=2] T,
           np.ndarray[np.complex64_t, ndim=2] Q=None,
           np.ndarray[np.complex64_t, ndim=2] Z=None,
           calc_ev=True):
    cdef l_int N, M, lwork, liwork, qiwork, info, ijob
    cdef l_logical wantq, wantz
    cdef float complex qwork, *q_ptr, *z_ptr
    cdef np.ndarray[np.complex64_t] alpha, beta, work
    cdef np.ndarray[l_int] iwork

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    alpha = np.empty(N, dtype = np.complex64)
    beta = np.empty(N, dtype = np.complex64)
    ijob = 0

    if Q is not None:
        wantq = 1
        q_ptr = <float complex *>Q.data
    else:
        wantq = 0
        q_ptr = NULL

    if Z is not None:
        wantz = 1
        z_ptr = <float complex *>Z.data
    else:
        wantz = 0
        z_ptr = NULL

    # workspace query
    lwork = -1
    liwork = -1
    f_lapack.ctgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <float complex *>S.data, &N,
                     <float complex *>T.data, &N,
                     <float complex *>alpha.data, <float complex *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     &qwork, &lwork, &qiwork, &liwork, &info)

    assert info == 0, "Argument error in ctgsen"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex64)
    liwork = qiwork
    iwork = np.empty(liwork, dtype = int_dtype)

    # Now the real calculation
    f_lapack.ctgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <float complex *>S.data, &N,
                     <float complex *>T.data, &N,
                     <float complex *>alpha.data, <float complex *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     <float complex *>work.data, &lwork,
                     <l_int *>iwork.data, &liwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in ctgsen"

    return filter_args((True, True, Q is not None, Z is not None,
                        calc_ev, calc_ev),
                       (S, T, Q, Z, alpha, beta))


def ztgsen(np.ndarray[l_logical] select,
           np.ndarray[np.complex128_t, ndim=2] S,
           np.ndarray[np.complex128_t, ndim=2] T,
           np.ndarray[np.complex128_t, ndim=2] Q=None,
           np.ndarray[np.complex128_t, ndim=2] Z=None,
           calc_ev=True):
    cdef l_int N, M, lwork, liwork, qiwork, info, ijob
    cdef l_logical wantq, wantz
    cdef double complex qwork, *q_ptr, *z_ptr
    cdef np.ndarray[np.complex128_t] alpha, beta, work
    cdef np.ndarray[l_int] iwork

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    alpha = np.empty(N, dtype = np.complex128)
    beta = np.empty(N, dtype = np.complex128)
    ijob = 0

    if Q is not None:
        wantq = 1
        q_ptr = <double complex *>Q.data
    else:
        wantq = 0
        q_ptr = NULL

    if Z is not None:
        wantz = 1
        z_ptr = <double complex *>Z.data
    else:
        wantz = 0
        z_ptr = NULL

    # workspace query
    lwork = -1
    liwork = -1
    f_lapack.ztgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <double complex *>S.data, &N,
                     <double complex *>T.data, &N,
                     <double complex *>alpha.data, <double complex *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     &qwork, &lwork, &qiwork, &liwork, &info)

    assert info == 0, "Argument error in ztgsen"

    lwork = <int>qwork.real
    work = np.empty(lwork, dtype = np.complex128)
    liwork = qiwork
    iwork = np.empty(liwork, dtype = int_dtype)

    # Now the real calculation
    f_lapack.ztgsen_(&ijob, &wantq, &wantz, <l_logical *>select.data,
                     &N, <double complex *>S.data, &N,
                     <double complex *>T.data, &N,
                     <double complex *>alpha.data, <double complex *>beta.data,
                     q_ptr, &N, z_ptr, &N, &M, NULL, NULL, NULL,
                     <double complex *>work.data, &lwork,
                     <l_int *>iwork.data, &liwork, &info)

    if info > 0:
        raise LinAlgError("Reordering failed; problem is very ill-conditioned")

    assert info == 0, "Argument error in ztgsen"

    return filter_args((True, True, Q is not None, Z is not None,
                        calc_ev, calc_ev),
                       (S, T, Q, Z, alpha, beta))


# xTGEVC
def stgevc(np.ndarray[np.float32_t, ndim=2] S,
           np.ndarray[np.float32_t, ndim=2] T,
           np.ndarray[np.float32_t, ndim=2] Q=None,
           np.ndarray[np.float32_t, ndim=2] Z=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.float32_t, ndim=2] vl_r, vr_r
    cdef float *vl_r_ptr, *vr_r_ptr
    cdef np.ndarray[l_logical] select_cpy
    cdef l_logical *select_ptr
    cdef np.ndarray[np.float32_t] work

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    work = np.empty(6*N, dtype = np.float32)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    backtr = False

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        # Correct for possible additional storage if a single complex
        # eigenvalue is selected.
        # For that: Figure out the positions of the 2x2 blocks.
        cmplxindx = np.diagonal(S, -1).nonzero()[0]
        for i in cmplxindx:
            if bool(select[i]) != bool(select[i+1]):
                MM += 1

        # select is overwritten in stgevc
        select_cpy = np.array(select, dtype = f_lapack.l_logical_dtype,
                              order = 'F')
        select_ptr = <l_logical *>select_cpy.data
    else:
        MM = N
        select_ptr = NULL
        if ((left and right and Q is not None and Z is not None) or
            (left and not right and Q is not None) or
            (right and not left and Z is not None)):
            howmny = "B"
            backtr = True
        else:
            howmny = "A"

    if left:
        if backtr:
            vl_r = Q
        else:
            vl_r = np.empty((N, MM), dtype = np.float32, order='F')
        vl_r_ptr = <float *>vl_r.data
    else:
        vl_r_ptr = NULL

    if right:
        if backtr:
            vr_r = Z
        else:
            vr_r = np.empty((N, MM), dtype = np.float32, order='F')
        vr_r_ptr = <float *>vr_r.data
    else:
        vr_r_ptr = NULL

    f_lapack.stgevc_(side, howmny, select_ptr,
                     &N, <float *>S.data, &N,
                     <float *>T.data, &N,
                     vl_r_ptr, &N, vr_r_ptr, &N, &MM, &M,
                     <float *>work.data, &info)

    assert info == 0, "Argument error in stgevc"
    assert MM == M, "Unexpected number of eigenvectors returned in stgevc"

    if not backtr:
        if left:
            vl_r = np.asfortranarray(np.dot(Q, vl_r))
        if right:
            vr_r = np.asfortranarray(np.dot(Z, vr_r))

    # If there are complex eigenvalues, we need to postprocess the eigenvectors
    if np.diagonal(S, -1).nonzero()[0].size:
        if left:
            vl = txevc_postprocess(np.complex64, S, vl_r, select)
        if right:
            vr = txevc_postprocess(np.complex64, S, vr_r, select)
    else:
        if left:
            vl = vl_r
        if right:
            vr = vr_r

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


def dtgevc(np.ndarray[np.float64_t, ndim=2] S,
           np.ndarray[np.float64_t, ndim=2] T,
           np.ndarray[np.float64_t, ndim=2] Q=None,
           np.ndarray[np.float64_t, ndim=2] Z=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.float64_t, ndim=2] vl_r, vr_r
    cdef double *vl_r_ptr, *vr_r_ptr
    cdef np.ndarray[l_logical] select_cpy
    cdef l_logical *select_ptr
    cdef np.ndarray[np.float64_t] work

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    work = np.empty(6*N, dtype = np.float64)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    backtr = False

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        # Correct for possible additional storage if a single complex
        # eigenvalue is selected.
        # For that: Figure out the positions of the 2x2 blocks.
        cmplxindx = np.diagonal(S, -1).nonzero()[0]
        for i in cmplxindx:
            if bool(select[i]) != bool(select[i+1]):
                MM += 1

        # select is overwritten in dtgevc
        select_cpy = np.array(select, dtype = f_lapack.l_logical_dtype,
                              order = 'F')
        select_ptr = <l_logical *>select_cpy.data
    else:
        MM = N
        select_ptr = NULL
        if ((left and right and Q is not None and Z is not None) or
            (left and not right and Q is not None) or
            (right and not left and Z is not None)):
            howmny = "B"
            backtr = True
        else:
            howmny = "A"

    if left:
        if backtr:
            vl_r = Q
        else:
            vl_r = np.empty((N, MM), dtype = np.float64, order='F')
        vl_r_ptr = <double *>vl_r.data
    else:
        vl_r_ptr = NULL

    if right:
        if backtr:
            vr_r = Z
        else:
            vr_r = np.empty((N, MM), dtype = np.float64, order='F')
        vr_r_ptr = <double *>vr_r.data
    else:
        vr_r_ptr = NULL

    f_lapack.dtgevc_(side, howmny, select_ptr,
                     &N, <double *>S.data, &N,
                     <double *>T.data, &N,
                     vl_r_ptr, &N, vr_r_ptr, &N, &MM, &M,
                     <double *>work.data, &info)

    assert info == 0, "Argument error in dtgevc"
    assert MM == M, "Unexpected number of eigenvectors returned in dtgevc"

    if not backtr:
        if left:
            vl_r = np.asfortranarray(np.dot(Q, vl_r))
        if right:
            vr_r = np.asfortranarray(np.dot(Z, vr_r))

    # If there are complex eigenvalues, we need to postprocess the
    # eigenvectors.
    if np.diagonal(S, -1).nonzero()[0].size:
        if left:
            vl = txevc_postprocess(np.complex128, S, vl_r, select)
        if right:
            vr = txevc_postprocess(np.complex128, S, vr_r, select)
    else:
        if left:
            vl = vl_r
        if right:
            vr = vr_r

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


def ctgevc(np.ndarray[np.complex64_t, ndim=2] S,
           np.ndarray[np.complex64_t, ndim=2] T,
           np.ndarray[np.complex64_t, ndim=2] Q=None,
           np.ndarray[np.complex64_t, ndim=2] Z=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.complex64_t, ndim=2] vl, vr
    cdef float complex *vl_ptr, *vr_ptr
    cdef l_logical *select_ptr
    cdef np.ndarray[np.complex64_t] work
    cdef np.ndarray[np.float32_t] rwork

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    work = np.empty(2*N, dtype = np.complex64)
    rwork = np.empty(2*N, dtype = np.float32)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    backtr = False

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        select_ptr = <l_logical *>select.data
    else:
        MM = N
        select_ptr = NULL
        if ((left and right and Q is not None and Z is not None) or
            (left and not right and Q is not None) or
            (right and not left and Z is not None)):
            howmny = "B"
            backtr = True
        else:
            howmny = "A"

    if left:
        if backtr:
            vl = Q
        else:
            vl = np.empty((N, MM), dtype = np.complex64, order='F')
        vl_ptr = <float complex *>vl.data
    else:
        vl_ptr = NULL

    if right:
        if backtr:
            vr = Z
        else:
            vr = np.empty((N, MM), dtype = np.complex64, order='F')
        vr_ptr = <float complex *>vr.data
    else:
        vr_ptr = NULL

    f_lapack.ctgevc_(side, howmny, select_ptr,
                     &N, <float complex *>S.data, &N,
                     <float complex *>T.data, &N,
                     vl_ptr, &N, vr_ptr, &N, &MM, &M,
                     <float complex *>work.data, <float *>rwork.data, &info)

    assert info == 0, "Argument error in ctgevc"
    assert MM == M, "Unexpected number of eigenvectors returned in ctgevc"

    if not backtr:
        if left:
            vl = np.asfortranarray(np.dot(Q, vl))
        if right:
            vr = np.asfortranarray(np.dot(Z, vr))

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


def ztgevc(np.ndarray[np.complex128_t, ndim=2] S,
           np.ndarray[np.complex128_t, ndim=2] T,
           np.ndarray[np.complex128_t, ndim=2] Q=None,
           np.ndarray[np.complex128_t, ndim=2] Z=None,
           np.ndarray[l_logical] select=None,
           left=False, right=True):
    cdef l_int N, info, M, MM
    cdef char *side, *howmny
    cdef np.ndarray[np.complex128_t, ndim=2] vl, vr
    cdef double complex *vl_ptr, *vr_ptr
    cdef l_logical *select_ptr
    cdef np.ndarray[np.complex128_t] work
    cdef np.ndarray[np.float64_t] rwork

    assert_fortran_mat(S, T, Q, Z)

    N = S.shape[0]
    work = np.empty(2*N, dtype = np.complex128)
    rwork = np.empty(2*N, dtype = np.float64)

    if left and right:
        side = "B"
    elif left:
        side = "L"
    elif right:
        side = "R"
    else:
        return

    backtr = False

    if select is not None:
        howmny = "S"
        MM = select.nonzero()[0].size
        select_ptr = <l_logical *>select.data
    else:
        MM = N
        select_ptr = NULL
        if ((left and right and Q is not None and Z is not None) or
            (left and not right and Q is not None) or
            (right and not left and Z is not None)):
            howmny = "B"
            backtr = True
        else:
            howmny = "A"

    if left:
        if backtr:
            vl = Q
        else:
            vl = np.empty((N, MM), dtype = np.complex128, order='F')
        vl_ptr = <double complex *>vl.data
    else:
        vl_ptr = NULL

    if right:
        if backtr:
            vr = Z
        else:
            vr = np.empty((N, MM), dtype = np.complex128, order='F')
        vr_ptr = <double complex *>vr.data
    else:
        vr_ptr = NULL

    f_lapack.ztgevc_(side, howmny, select_ptr,
                     &N, <double complex *>S.data, &N,
                     <double complex *>T.data, &N,
                     vl_ptr, &N, vr_ptr, &N, &MM, &M,
                     <double complex *>work.data, <double *>rwork.data, &info)

    assert info == 0, "Argument error in ztgevc"
    assert MM == M, "Unexpected number of eigenvectors returned in ztgevc"

    if not backtr:
        if left:
            vl = np.asfortranarray(np.dot(Q, vl))
        if right:
            vr = np.asfortranarray(np.dot(Z, vr))

    if left and right:
        return (vl, vr)
    elif left:
        return vl
    else:
        return vr


def prepare_for_lapack(overwrite, *args):
    """Convert arrays to Fortran format.

    This function takes a number of array objects in `args` and converts them
    to a format that can be directly passed to a Fortran function (Fortran
    contiguous NumPy array). If the arrays have different data type, they
    converted arrays are cast to a common compatible data type (one of NumPy's
    `float32`, `float64`, `complex64`, `complex128` data types).

    If `overwrite` is ``False``, an NumPy array that would already be in the
    correct format (Fortran contiguous, right data type) is neverthelessed
    copied. (Hence, overwrite = True does not imply that acting on the
    converted array in the return values will overwrite the original array in
    all cases -- it does only so if the original array was already in the
    correct format. The conversions require copying. In fact, that's the same
    behavior as in SciPy, it's just not explicitly stated there)

    If an argument is ``None``, it is just passed through and not used to
    determine the proper LAPACK type.

    `prepare_for_lapack` returns a character indicating the proper LAPACK data
    type ('s', 'd', 'c', 'z') and a list of properly converted arrays.
    """

    # Make sure we have NumPy arrays
    mats = [None]*len(args)
    for i in xrange(len(args)):
        if args[i] is not None:
            arr = np.asanyarray(args[i])
            if not np.issubdtype(arr.dtype, np.number):
                raise ValueError("Argument cannot be interpreted "
                                 "as a numeric array")

            mats[i] = (arr, arr is not args[i] or overwrite)
        else:
            mats[i] = (None, True)

    # First figure out common dtype
    # Note: The return type of common_type is guaranteed to be a floating point
    #       kind.
    dtype = np.common_type(*[arr for arr, ovwrt in mats if arr is not None])

    if dtype == np.float32:
        lapacktype = 's'
    elif dtype == np.float64:
        lapacktype = 'd'
    elif dtype == np.complex64:
        lapacktype = 'c'
    elif dtype == np.complex128:
        lapacktype = 'z'
    else:
        raise AssertionError("Unexpected data type from common_type")

    ret = [ lapacktype ]
    for npmat, ovwrt in mats:
        # Now make sure that the array is contiguous, and copy if necessary.
        if npmat is not None:
            if npmat.ndim == 2:
                if not npmat.flags["F_CONTIGUOUS"]:
                    npmat = np.asfortranarray(npmat, dtype = dtype)
                elif npmat.dtype != dtype:
                    npmat = npmat.astype(dtype)
                elif not ovwrt:
                    # ugly here: copy makes always C-array, no way to tell it
                    # to make a Fortran array.
                    npmat = np.asfortranarray(npmat.copy())
            elif npmat.ndim == 1:
                if not npmat.flags["C_CONTIGUOUS"]:
                    npmat = np.ascontiguousarray(npmat, dtype = dtype)
                elif npmat.dtype != dtype:
                    npmat = npmat.astype(dtype)
                elif not ovwrt:
                    npmat = np.asfortranarray(npmat.copy())
            else:
                raise ValueError("Dimensionality of array is not 1 or 2")

        ret.append(npmat)

    return tuple(ret)



# Tutorial 2.3.2. Spatially dependent values through functions
# ============================================================
#
# Physics background
# ------------------
#  transmission through a quantum well
#
# Kwant features highlighted
# --------------------------
#  - Functions as values in Builder

import kwant

# For plotting
from matplotlib import pyplot


#HIDDEN_BEGIN_ehso
def make_system(a=1, t=1.0, W=10, L=30, L_well=10):
    # Start with an empty tight-binding system and a single square lattice.
    # `a` is the lattice constant (by default set to 1 for simplicity.
    lat = kwant.lattice.square(a)

    sys = kwant.Builder()

    #### Define the scattering region. ####
    # Potential profile
    def potential(site, pot):
        (x, y) = site.pos
        if (L - L_well) / 2 < x < (L + L_well) / 2:
            return pot
        else:
            return 0
#HIDDEN_END_ehso

#HIDDEN_BEGIN_coid
    def onsite(site, pot=0):
        return 4 * t + potential(site, pot)

    sys[(lat(x, y) for x in range(L) for y in range(W))] = onsite
    sys[lat.neighbors()] = -t
#HIDDEN_END_coid

    #### Define and attach the leads. ####
    lead = kwant.Builder(kwant.TranslationalSymmetry((-a, 0)))
    lead[(lat(0, j) for j in xrange(W))] = 4 * t
    lead[lat.neighbors()] = -t
    sys.attach_lead(lead)
    sys.attach_lead(lead.reversed())

    return sys


def plot_conductance(sys, energy, welldepths):
#HIDDEN_BEGIN_sqvr

    # Compute conductance
    data = []
    for welldepth in welldepths:
        smatrix = kwant.smatrix(sys, energy, args=[-welldepth])
        data.append(smatrix.transmission(1, 0))

    pyplot.figure()
    pyplot.plot(welldepths, data)
    pyplot.xlabel("well depth [t]")
    pyplot.ylabel("conductance [e^2/h]")
    pyplot.show()
#HIDDEN_END_sqvr


def main():
    sys = make_system()

    # Check that the system looks as intended.
    kwant.plot(sys)

    # Finalize the system.
    sys = sys.finalized()

    # We should see conductance steps.
    plot_conductance(sys, energy=0.2,
                     welldepths=[0.01 * i for i in xrange(100)])


# Call the main function if the script gets executed (as opposed to imported).
# See <http://docs.python.org/library/__main__.html>.
if __name__ == '__main__':
    main()



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

import tempfile
import warnings
import nose
import kwant
from kwant import plotter
if plotter.mpl_enabled:
    from mpl_toolkits import mplot3d
    from matplotlib import pyplot


def test_importable_without_matplotlib():
    prefix, sep, suffix = plotter.__file__.rpartition('.')
    if suffix == 'pyc':
        suffix = 'py'
    assert suffix == 'py'
    fname = sep.join((prefix, suffix))
    with open(fname) as f:
        code = f.read()
    code = code.replace('from . import', 'from kwant import')
    code = code.replace('matplotlib', 'totalblimp')

    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        exec code               # Trigger the warning.
        nose.tools.assert_equal(len(w), 1)
        assert issubclass(w[0].category, RuntimeWarning)
        assert "only iterator-providing functions" in str(w[0].message)


def sys_2d(W=3, r1=3, r2=8):
    a = 1
    t = 1.0
    lat = kwant.lattice.square(a)
    sys = kwant.Builder()

    def ring(pos):
        (x, y) = pos
        rsq = x ** 2 + y ** 2
        return r1 ** 2 < rsq < r2 ** 2

    sys[lat.shape(ring, (0, r1 + 1))] = 4 * t
    sys[lat.neighbors()] = -t
    sym_lead0 = kwant.TranslationalSymmetry(lat.vec((-1, 0)))
    lead0 = kwant.Builder(sym_lead0)
    lead2 = kwant.Builder(sym_lead0)

    lead_shape = lambda pos: (-W / 2 < pos[1] < W / 2)

    lead0[lat.shape(lead_shape, (0, 0))] = 4 * t
    lead2[lat.shape(lead_shape, (0, 0))] = 4 * t
    sys.attach_lead(lead2)
    lead0[lat.neighbors()] = - t
    lead1 = lead0.reversed()
    sys.attach_lead(lead0)
    sys.attach_lead(lead1)
    return sys


def sys_3d(W=3, r1=2, r2=4, a=1, t=1.0):
    lat = kwant.lattice.general(((a, 0, 0), (0, a, 0), (0, 0, a)))
    sys = kwant.Builder()

    def ring(pos):
        (x, y, z) = pos
        rsq = x ** 2 + y ** 2
        return (r1 ** 2 < rsq < r2 ** 2) and abs(z) < 2
    sys[lat.shape(ring, (0, -r2 + 1, 0))] = 4 * t
    sys[lat.neighbors()] = - t
    sym_lead0 = kwant.TranslationalSymmetry(lat.vec((-1, 0, 0)))
    lead0 = kwant.Builder(sym_lead0)

    lead_shape = lambda pos: (-W / 2 < pos[1] < W / 2) and abs(pos[2]) < 2

    lead0[lat.shape(lead_shape, (0, 0, 0))] = 4 * t
    lead0[lat.neighbors()] = - t
    lead1 = lead0.reversed()
    sys.attach_lead(lead0)
    sys.attach_lead(lead1)
    return sys


def test_plot():
    plot = plotter.plot
    if not plotter.mpl_enabled:
        raise nose.SkipTest
    sys2d = sys_2d()
    sys3d = sys_3d()
    color_opts = ['k', (lambda site: site.tag[0]),
                  lambda site: (abs(site.tag[0] / 100),
                                abs(site.tag[1] / 100), 0)]
    with tempfile.TemporaryFile('w+b') as out:
        for color in color_opts:
            for sys in (sys2d, sys3d):
                fig = plot(sys, site_color=color, cmap='binary', file=out)
                if color != 'k' and \
                   isinstance(color(iter(sys2d.sites()).next()), float):
                    assert fig.axes[0].collections[0].get_array() is not None
                assert len(fig.axes[0].collections) == (8 if sys is sys2d else
                                                        6)
        color_opts = ['k', (lambda site, site2: site.tag[0]),
                      lambda site, site2: (abs(site.tag[0] / 100),
                                           abs(site.tag[1] / 100), 0)]
        for color in color_opts:
            for sys in (sys2d, sys3d):
                fig = plot(sys2d, hop_color=color, cmap='binary', file=out,
                           fig_size=(2, 10), dpi=30)
                if color != 'k' and isinstance(color(iter(sys2d.sites()).next(),
                                                          None), float):
                    assert fig.axes[0].collections[1].get_array() is not None

        assert isinstance(plot(sys3d, file=out).axes[0], mplot3d.axes3d.Axes3D)

        sys2d.leads = []
        plot(sys2d, file=out)
        del sys2d[list(sys2d.hoppings())]
        plot(sys2d, file=out)

        plot(sys3d, file=out)
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            plot(sys2d.finalized(), file=out)


def test_map():
    if not plotter.mpl_enabled:
        raise nose.SkipTest
    sys = sys_2d()
    with tempfile.TemporaryFile('w+b') as out:
        plotter.map(sys, lambda site: site.tag[0], file=out,
                          method='linear', a=4, oversampling=4, cmap='flag')
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            plotter.map(sys.finalized(), xrange(len(sys.sites())),
                              file=out)
        nose.tools.assert_raises(ValueError, plotter.map, sys,
                                 xrange(len(sys.sites())), file=out)



#!/usr/bin/env python

# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

CONFIG_FILE = 'build.conf'
README_FILE = 'README'
STATIC_VERSION_FILE = 'kwant/_static_version.py'
REQUIRED_CYTHON_VERSION = (0, 17, 1)
NO_CYTHON_OPTION = '--no-cython'
TUT_DIR = 'tutorial'
TUT_GLOB = 'doc/source/tutorial/*.py'
TUT_HIDDEN_PREFIX = '#HIDDEN'

import sys
import re
import os
import glob
import subprocess
import ConfigParser
from distutils.core import setup, Extension, Command
from distutils.util import get_platform
from distutils.errors import DistutilsError, DistutilsModuleError, \
    CCompilerError
from distutils.command.build import build as distutils_build
from distutils.command.sdist import sdist as distutils_sdist
import numpy

try:
    import Cython
except:
    cython_version = ()
else:
    match = re.match('([0-9.]*)(.*)', Cython.__version__)
    cython_version = [int(n) for n in match.group(1).split('.')]
    # Decrease version if the version string contains a suffix.
    if match.group(2):
        while cython_version[-1] == 0:
            cython_version.pop()
        cython_version[-1] -= 1
    cython_version = tuple(cython_version)

try:
    sys.argv.remove(NO_CYTHON_OPTION)
    cythonize = False
except ValueError:
    cythonize = True

if cythonize and cython_version:
    from Cython.Distutils import build_ext
else:
    from distutils.command.build_ext import build_ext

distr_root = os.path.dirname(os.path.abspath(__file__))


class kwant_build_ext(build_ext):
    def run(self):
        if not config_file_present:
            # Create an empty config file if none is present so that the
            # extensions will not be rebuilt each time.  Only depending on the
            # config file if it is present would make it impossible to detect a
            # necessary rebuild due to a deleted config file.
            with open(CONFIG_FILE, 'w') as f:
                f.write('# Created by setup.py - feel free to modify.\n')

        try:
            build_ext.run(self)
        except (DistutilsError, CCompilerError):
            print >>sys.stderr, \
"""{0}
The compilation of Kwant has failed.  Please examine the error message
above and consult the installation instructions in README.
You might have to customize {1}.
{0}
Build configuration was:
{2}
{0}""".format('*' * 70, CONFIG_FILE, build_summary)
            raise
        print '**************** Build summary ****************'
        print build_summary


class build_tut(Command):
    description = "build the tutorial scripts"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if not os.path.exists(TUT_DIR):
            os.mkdir(TUT_DIR)
        for in_fname in glob.glob(TUT_GLOB):
            out_fname = os.path.join(TUT_DIR, os.path.basename(in_fname))
            with open(in_fname) as in_file:
                with open(out_fname, 'w') as out_file:
                    for line in in_file:
                        if not line.startswith(TUT_HIDDEN_PREFIX):
                            out_file.write(line)


# Our version of the "build" command also makes sure the tutorial is made.
# Even though the tutorial is not necessary for installation, and "build" is
# supposed to make everything needed to install, this is a robust way to ensure
# that the tutorial is present.
class kwant_build(distutils_build):
    sub_commands = [('build_tut', None)] + distutils_build.sub_commands


class test(Command):
    description = "build, then run the unit tests"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            from nose.core import run
        except ImportError:
            raise DistutilsModuleError('nose <http://nose.readthedocs.org/> '
                                       'is needed to run the tests')
        self.run_command('build')
        major, minor = sys.version_info[:2]
        lib_dir = "build/lib.{0}-{1}.{2}".format(get_platform(), major, minor)
        print '**************** Tests ****************'
        if not run(argv=[__file__, '-v', lib_dir]):
            raise DistutilsError('at least one of the tests failed')


def git_lsfiles():
    try:
        p = subprocess.Popen(['git', 'ls-files'], cwd=distr_root,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except OSError:
        return

    if p.wait() != 0:
        return
    return p.communicate()[0].split('\n')[:-1]


# Make the command "sdist" depend on "build".  This verifies that the
# distribution in the current state actually builds.  It also makes sure that
# the Cython-made C files and the tutorial will be included in the source
# distribution and that they will be up-to-date.
class kwant_sdist(distutils_sdist):
    sub_commands = [('build', None)] + distutils_sdist.sub_commands

    def run(self):
        names = git_lsfiles()
        trustworthy = True
        if names is None:
            # Check that MANIFEST exists and has not been generated by
            # distutils.
            try:
                with open(distr_root + '/MANIFEST', 'r') as f:
                    line = f.read()
            except IOError:
                print >>sys.stderr, "error: MANIFEST file is missing and " \
                    "Git is not available to regenerate it."
                exit(1)
            trustworthy = not line.strip().startswith('#')
        else:
            # Generate MANIFEST file.
            with open(distr_root + '/MANIFEST', 'w') as f:
                for name in names:
                    a, sep, b = name.rpartition('/')
                    if b == '.gitignore':
                        continue
                    stem, dot, extension = b.rpartition('.')
                    if extension == 'pyx':
                        f.write(''.join([a, sep, stem, dot, 'c', '\n']))
                    f.write(name + '\n')
                f.write(STATIC_VERSION_FILE + '\n')
                f.write('MANIFEST\n')

        distutils_sdist.run(self)

        if names is None:
            print >>sys.stderr, \
    """**************** Warning ****************
Git was not available for re-generating the MANIFEST file (the list of file
names to be included in the source distribution).  The old MANIFEST was used."""

        if not trustworthy:
            print >>sys.stderr, \
    """**************** Warning ****************
The existing MANIFEST file seems to have been generated by distutils (it begins
with a comment).  It may well be incomplete."""


# This is an exact copy of the function from kwant/version.py.  We can't import
# it here (because Kwant is not yet built when this scipt is run), so we just
# include a copy.
def get_version_from_git():
    try:
        p = subprocess.Popen(['git', 'rev-parse', '--show-toplevel'],
                             cwd=distr_root,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except OSError:
        return
    if p.wait() != 0:
        return
    # TODO: use os.path.samefile once we depend on Python >= 3.3.
    if os.path.normpath(p.communicate()[0].rstrip('\n')) != distr_root:
        # The top-level directory of the current Git repository is not the same
        # as the root directory of the Kwant distribution: do not extract the
        # version from Git.
        return

    # git describe --first-parent does not take into account tags from branches
    # that were merged-in.
    for opts in [['--first-parent'], []]:
        try:
            p = subprocess.Popen(['git', 'describe'] + opts, cwd=distr_root,
                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        except OSError:
            return
        if p.wait() == 0:
            break
    else:
        return
    version = p.communicate()[0].rstrip('\n')

    if version[0] == 'v':
        version = version[1:]

    try:
        p = subprocess.Popen(['git', 'diff', '--quiet'], cwd=distr_root)
    except OSError:
        version += '-confused'  # This should never happen.
    else:
        if p.wait() == 1:
            version += '-dirty'
    return version


def get_static_version():
    """Return the version as recorded inside the source code."""
    try:
        with open(STATIC_VERSION_FILE) as f:
            contents = f.read()
            assert contents[:11] == "version = '"
            assert contents[-2:] == "'\n"
            return contents[11:-2]
    except:
        return None


def version():
    """Determine the version of Kwant.  Return it and save it in a file."""
    git_version = get_version_from_git()
    static_version = get_static_version()
    if git_version is not None:
        version = git_version
        if static_version != git_version:
            with open(STATIC_VERSION_FILE, 'w') as f:
                f.write("version = '%s'\n" % version)
    elif static_version is not None:
        version = static_version
    else:
        version = 'unknown'
    return version


def long_description():
    text = []
    try:
        with open(README_FILE) as f:
            for line in f:
                if line == "\n":
                    break
                text.append(line.rstrip())
    except:
        return ''
    return '\n'.join(text)


def packages():
    return [root.replace('/', '.')
            for root, dnames, fnames in os.walk('kwant')
            if '__init__.py' in fnames or root.endswith('/tests')]


def search_mumps():
    """Return the configuration for MUMPS if it is available in a known way.

    This is known to work with the MUMPS provided by the Debian package
    libmumps-scotch-dev."""

    libs = ['zmumps_scotch', 'mumps_common_scotch', 'pord', 'mpiseq_scotch',
            'gfortran']

    cmd = ['gcc']
    cmd.extend(['-l' + lib for lib in libs])
    cmd.extend(['-o/dev/null', '-xc', '-'])
    try:
        p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
    except OSError:
        pass
    else:
        p.communicate(input='int main() {}\n')
        if p.wait() == 0:
            return {'libraries': libs}
    return {}


def extensions():
    """Return a list of tuples (args, kwrds) to be passed to
    Extension. possibly after replacing ".pyx" with ".c" if Cython is not to be
    used."""

    global build_summary, config_file_present
    build_summary = []

    #### Add components of Kwant without external compile-time dependencies.
    result = [
        (['kwant._system', ['kwant/_system.pyx']],
         {'include_dirs': ['kwant/graph']}),
        (['kwant.graph.core', ['kwant/graph/core.pyx']],
         {'depends': ['kwant/graph/core.pxd', 'kwant/graph/defs.h',
                      'kwant/graph/defs.pxd']}),
        (['kwant.graph.utils', ['kwant/graph/utils.pyx']],
         {'depends': ['kwant/graph/defs.h', 'kwant/graph/defs.pxd',
                      'kwant/graph/core.pxd']}),
        (['kwant.graph.slicer', ['kwant/graph/slicer.pyx',
                                 'kwant/graph/c_slicer/partitioner.cc',
                                 'kwant/graph/c_slicer/slicer.cc']],
         {'depends': ['kwant/graph/defs.h', 'kwant/graph/defs.pxd',
                      'kwant/graph/core.pxd',
                      'kwant/graph/c_slicer.pxd',
                      'kwant/graph/c_slicer/bucket_list.h',
                      'kwant/graph/c_slicer/graphwrap.h',
                      'kwant/graph/c_slicer/partitioner.h',
                      'kwant/graph/c_slicer/slicer.h']})]

    #### Add components of Kwant with external compile-time dependencies.
    config = ConfigParser.ConfigParser()
    try:
        with open(CONFIG_FILE) as f:
            config.readfp(f)
    except IOError:
        config_file_present = False
    else:
        config_file_present = True

    kwrds_by_section = {}
    for section in config.sections():
        kwrds_by_section[section] = kwrds = {}
        for name, value in config.items(section):
            kwrds[name] = value.split()

    # Setup LAPACK.
    lapack = kwrds_by_section.get('lapack')
    if lapack:
        build_summary.append('User-configured LAPACK and BLAS')
    else:
        lapack = {'libraries': ['lapack', 'blas']}
        build_summary.append('Default LAPACK and BLAS')
    kwrds = lapack.copy()
    kwrds.setdefault('depends', []).extend(
        [CONFIG_FILE, 'kwant/linalg/f_lapack.pxd'])
    result.append((['kwant.linalg.lapack', ['kwant/linalg/lapack.pyx']],
                   kwrds))

    # Setup MUMPS.
    kwrds = kwrds_by_section.get('mumps')
    if kwrds:
        build_summary.append('User-configured MUMPS')
    else:
        kwrds = search_mumps()
        if kwrds:
            build_summary.append('Auto-configured MUMPS')
    if kwrds:
        for name, value in lapack.iteritems():
            kwrds.setdefault(name, []).extend(value)
        kwrds.setdefault('depends', []).extend(
            [CONFIG_FILE, 'kwant/linalg/cmumps.pxd'])
        result.append((['kwant.linalg._mumps', ['kwant/linalg/_mumps.pyx']],
                       kwrds))
    else:
        build_summary.append('No MUMPS support')

    build_summary = '\n'.join(build_summary)
    return result


def complain_cython_unavailable():
    assert not cythonize or cython_version < REQUIRED_CYTHON_VERSION
    if cythonize:
        msg = "Install Cython {0} or newer so it can be made or use a source " \
            "distribution of Kwant."
        ver = '.'.join(str(e) for e in REQUIRED_CYTHON_VERSION)
        print >>sys.stderr, msg.format(ver)
    else:
        print >>sys.stderr, "Run setup.py without", \
            NO_CYTHON_OPTION


def ext_modules(extensions):
    """Prepare the ext_modules argument for distutils' setup."""
    result = []
    for args, kwrds in extensions:
        if not cythonize or cython_version < REQUIRED_CYTHON_VERSION:
            if 'language' in kwrds:
                if kwrds['language'] == 'c':
                    ext = '.c'
                elif kwrds['language'] == 'c++':
                    ext = '.cpp'
                else:
                    print >>sys.stderr, 'Unknown language'
                    exit(1)
            else:
                ext = '.c'
            pyx_files = []
            cythonized_files = []
            sources = []
            for f in args[1]:
                if f[-4:] == '.pyx':
                    pyx_files.append(f)
                    f = f[:-4] + ext
                    cythonized_files.append(f)
                sources.append(f)
            args[1] = sources

            try:
                cythonized_oldest = min(os.stat(f).st_mtime
                                        for f in cythonized_files)
            except OSError:
                print >>sys.stderr, \
                    "error: Cython-generated file {0} is missing.".format(f)
                complain_cython_unavailable()
                exit(1)
            for f in pyx_files + kwrds.get('depends', []):
                if f == CONFIG_FILE:
                    # The config file is only a dependency for the compilation
                    # of the cythonized file, not for the cythonization.
                    continue
                if os.stat(f).st_mtime > cythonized_oldest:
                    msg = "error: {0} is newer than its source file, but "
                    if cythonize and not cython_version:
                        msg += "Cython is not installed."
                    elif cythonize:
                        msg += "the installed Cython is too old."
                    else:
                        msg += "Cython is not to be run."
                    print >>sys.stderr, msg.format(f)
                    complain_cython_unavailable()
                    exit(1)

        result.append(Extension(*args, **kwrds))

    return result


def main():
    setup(name='kwant',
          version=version(),
          author='C. W. Groth (CEA), M. Wimmer, A. R. Akhmerov, X. Waintal (CEA), and others',
          author_email='authors@kwant-project.org',
          description="Package for numerical quantum transport calculations.",
          long_description=long_description(),
          platforms=["Unix", "Linux", "Mac OS-X", "Windows"],
          url="http://kwant-project.org/",
          license="BSD",
          packages=packages(),
          cmdclass={'build': kwant_build,
                    'sdist': kwant_sdist,
                    'build_ext': kwant_build_ext,
                    'build_tut': build_tut,
                    'test': test},
          ext_modules=ext_modules(extensions()),
          include_dirs=[numpy.get_include()])

if __name__ == '__main__':
    main()



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from __future__ import division
import numpy as np
from kwant.linalg import lll

def test_lll():
    np.random.seed(1)
    for i in range(50):
        x = np.random.randint(4) + 1
        mat = np.random.randn(x, x + np.random.randint(2))
        c = 1.34 + .5 * np.random.rand()
        reduced_mat, coefs = lll.lll(mat)
        assert lll.is_c_reduced(reduced_mat, c)
        assert np.allclose(np.dot(mat.T, coefs), reduced_mat.T)


def test_cvp():
    np.random.seed(0)
    for i in range(10):
        mat = np.random.randn(4, 4)
        mat = lll.lll(mat)[0]
        for j in range(4):
            point = 50 * np.random.randn(4)
            assert np.array_equal(lll.cvp(point, mat, 10)[:3],
                                  lll.cvp(point, mat, 3))



# Tutorial 2.2.2. Transport through a quantum wire
# ================================================
#
# Physics background
# ------------------
#  Conductance of a quantum wire; subbands
#
# Kwant features highlighted
# --------------------------
#  - Builder for setting up transport systems easily
#  - Making scattering region and leads
#  - Using the simple sparse solver for computing Landauer conductance

from matplotlib import pyplot
#HIDDEN_BEGIN_dwhx
import kwant
#HIDDEN_END_dwhx

# First, define the tight-binding system

#HIDDEN_BEGIN_goiq
sys = kwant.Builder()
#HIDDEN_END_goiq

# Here, we are only working with square lattices
#HIDDEN_BEGIN_suwo
a = 1
lat = kwant.lattice.square(a)
#HIDDEN_END_suwo

#HIDDEN_BEGIN_zfvr
t = 1.0
W = 10
L = 30

# Define the scattering region

for i in xrange(L):
    for j in xrange(W):
        # On-site Hamiltonian
        sys[lat(i, j)] = 4 * t

        # Hopping in y-direction
        if j > 0:
            sys[lat(i, j), lat(i, j - 1)] = -t

        # Hopping in x-direction
        if i > 0:
            sys[lat(i, j), lat(i - 1, j)] = -t
#HIDDEN_END_zfvr

# Then, define and attach the leads:

# First the lead to the left
# (Note: TranslationalSymmetry takes a real-space vector)
#HIDDEN_BEGIN_xcmc
sym_left_lead = kwant.TranslationalSymmetry((-a, 0))
left_lead = kwant.Builder(sym_left_lead)
#HIDDEN_END_xcmc

#HIDDEN_BEGIN_ndez
for j in xrange(W):
    left_lead[lat(0, j)] = 4 * t
    if j > 0:
        left_lead[lat(0, j), lat(0, j - 1)] = -t
    left_lead[lat(1, j), lat(0, j)] = -t
#HIDDEN_END_ndez

#HIDDEN_BEGIN_fskr
sys.attach_lead(left_lead)
#HIDDEN_END_fskr

# Then the lead to the right
#HIDDEN_BEGIN_xhqc
sym_right_lead = kwant.TranslationalSymmetry((a, 0))
right_lead = kwant.Builder(sym_right_lead)

for j in xrange(W):
    right_lead[lat(0, j)] = 4 * t
    if j > 0:
        right_lead[lat(0, j), lat(0, j - 1)] = -t
    right_lead[lat(1, j), lat(0, j)] = -t

sys.attach_lead(right_lead)
#HIDDEN_END_xhqc

# Plot it, to make sure it's OK
#HIDDEN_BEGIN_wsgh
kwant.plot(sys)
#HIDDEN_END_wsgh

# Finalize the system
#HIDDEN_BEGIN_dngj
sys = sys.finalized()
#HIDDEN_END_dngj

# Now that we have the system, we can compute conductance
#HIDDEN_BEGIN_buzn
energies = []
data = []
for ie in xrange(100):
    energy = ie * 0.01

    # compute the scattering matrix at a given energy
    smatrix = kwant.smatrix(sys, energy)

    # compute the transmission probability from lead 0 to
    # lead 1
    energies.append(energy)
    data.append(smatrix.transmission(1, 0))
#HIDDEN_END_buzn

# Use matplotlib to write output
# We should see conductance steps
#HIDDEN_BEGIN_lliv
pyplot.figure()
pyplot.plot(energies, data)
pyplot.xlabel("energy [t]")
pyplot.ylabel("conductance [e^2/h]")
pyplot.show()
#HIDDEN_END_lliv



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from __future__ import division
from math import sqrt
import numpy as np
import tinyarray as ta
from nose.tools import assert_raises, assert_not_equal
from numpy.testing import assert_equal
from kwant import lattice, builder


def test_closest():
    np.random.seed(4)
    lat = lattice.general(((1, 0), (0.5, sqrt(3)/2)))
    for i in range(50):
        point = 20 * np.random.rand(2)
        closest = lat(*lat.closest(point)).pos
        assert np.linalg.norm(point - closest) <= 1 / sqrt(3)
    lat = lattice.general(np.random.randn(3, 3))
    for i in range(50):
        tag = np.random.randint(10, size=(3,))
        assert_equal(lat.closest(lat(*tag).pos), tag)


def test_general():
    for lat in (lattice.general(((1, 0), (0.5, 0.5))),
                lattice.general(((1, 0), (0.5, sqrt(3)/2)),
                                     ((0, 0), (0, 1/sqrt(3))))):
        for sl in lat.sublattices:
            tag = (-5, 33)
            site = sl(*tag)
            assert_equal(tag, sl.closest(site.pos))

    # Test 2D lattice with 1 vector.
    lat = lattice.general([[1, 0]])
    site = lat(0)
    assert_raises(ValueError, lat, 0, 1)


def test_neighbors():
    lat = lattice.honeycomb(1e-10)
    num_nth_nearest = [len(lat.neighbors(n)) for n in range(5)]
    assert num_nth_nearest == [2, 3, 6, 3, 6]
    lat = lattice.square(1e8)
    num_nth_nearest = [len(lat.neighbors(n)) for n in range(5)]
    assert num_nth_nearest == [1, 2, 2, 2, 4]
    lat = lattice.chain(1e-10)
    num_nth_nearest = [len(lat.neighbors(n)) for n in range(5)]
    assert num_nth_nearest == 5 * [1]


def test_shape():
    def in_circle(pos):
        return pos[0] ** 2 + pos[1] ** 2 < 3

    lat = lattice.honeycomb()
    sites = list(lat.shape(in_circle, (0, 0))())
    sites_alt = list()
    sl0, sl1 = lat.sublattices
    for x in xrange(-2, 3):
        for y in xrange(-2, 3):
            tag = (x, y)
            for site in (sl0(*tag), sl1(*tag)):
                if in_circle(site.pos):
                    sites_alt.append(site)
    assert len(sites) == len(sites_alt)
    assert_equal(set(sites), set(sites_alt))
    assert_raises(ValueError, lat.shape(in_circle, (10, 10))().next)

    # Check if narrow ribbons work.
    for period in (0, 1), (1, 0), (1, -1):
        vec = lat.vec(period)
        sym = lattice.TranslationalSymmetry(vec)
        def shape(pos):
            return abs(pos[0] * vec[1] - pos[1] * vec[0]) < 10
        sites = list(lat.shape(shape, (0, 0))(sym))
        assert len(sites) > 35


def test_wire():
    np.random.seed(5)
    vecs = np.random.randn(3, 3)
    vecs[0] = [1, 0, 0]
    center = np.random.randn(3)
    lat = lattice.general(vecs, np.random.randn(4, 3))
    sys = builder.Builder(lattice.TranslationalSymmetry((2, 0, 0)))
    def wire_shape(pos):
        pos = np.array(pos)
        return np.linalg.norm(pos[1:] - center[1:])**2 <= 8.6**2
    sys[lat.shape(wire_shape, center)] = 0
    sites2 = set(sys.sites())
    sys = builder.Builder(lattice.TranslationalSymmetry((2, 0, 0)))
    sys[lat.wire(center, 8.6)] = 1
    sites1 = set(sys.sites())
    assert_equal(sites1, sites2)


def test_translational_symmetry():
    ts = lattice.TranslationalSymmetry
    f2 = lattice.general(np.identity(2))
    f3 = lattice.general(np.identity(3))
    shifted = lambda site, delta: site.family(*ta.add(site.tag, delta))

    assert_raises(ValueError, ts, (0, 0, 4), (0, 5, 0), (0, 0, 2))
    sym = ts((3.3, 0))
    assert_raises(ValueError, sym.add_site_family, f2)

    # Test lattices with dimension smaller than dimension of space.
    f2in3 = lattice.general([[4, 4, 0], [4, -4, 0]])
    sym = ts((8, 0, 0))
    sym.add_site_family(f2in3)
    sym = ts((8, 0, 1))
    assert_raises(ValueError, sym.add_site_family, f2in3)

    # Test automatic fill-in of transverse vectors.
    sym = ts((1, 2))
    sym.add_site_family(f2)
    assert_not_equal(sym.site_family_data[f2][2], 0)
    sym = ts((1, 0, 2), (3, 0, 2))
    sym.add_site_family(f3)
    assert_not_equal(sym.site_family_data[f3][2], 0)

    transl_vecs = np.array([[10, 0], [7, 7]], dtype=int)
    sym = ts(*transl_vecs)
    assert_equal(sym.num_directions, 2)
    sym2 = ts(*transl_vecs[: 1, :])
    sym2.add_site_family(f2, transl_vecs[1:, :])
    for site in [f2(0, 0), f2(4, 0), f2(2, 1), f2(5, 5), f2(15, 6)]:
        assert sym.in_fd(site)
        assert sym2.in_fd(site)
        assert_equal(sym.which(site), (0, 0))
        assert_equal(sym2.which(site), (0,))
        for v in [(1, 0), (0, 1), (-1, 0), (0, -1), (5, 10), (-111, 573)]:
            site2 = shifted(site, np.dot(v, transl_vecs))
            assert not sym.in_fd(site2)
            assert (v[0] != 0) != sym2.in_fd(site2)
            assert_equal(sym.to_fd(site2), site)
            assert (v[1] == 0) == (sym2.to_fd(site2) == site)
            assert_equal(sym.which(site2), v)
            assert_equal(sym2.which(site2), v[:1])

            for hop in [(0, 0), (100, 0), (0, 5), (-2134, 3213)]:
                assert_equal(sym.to_fd(site2, shifted(site2, hop)),
                             (site, shifted(site, hop)))

    # Test act for hoppings belonging to different lattices.
    f2p = lattice.general(2 * np.identity(2))
    sym = ts(*(2 * np.identity(2)))
    assert sym.act((1, 1), f2(0, 0), f2p(0, 0)) == (f2(2, 2), f2p(1, 1))
    assert sym.act((1, 1), f2p(0, 0), f2(0, 0)) == (f2p(1, 1), f2(2, 2))

    # Test add_site_family on random lattices and symmetries by ensuring that
    # it's possible to add site groups that are compatible with a randomly
    # generated symmetry with proper vectors.
    np.random.seed(30)
    vec = np.random.randn(3, 5)
    lat = lattice.general(vec)
    total = 0
    for k in range(1, 4):
        for i in range(10):
            sym_vec = np.random.randint(-10, 10, size=(k, 3))
            if np.linalg.matrix_rank(sym_vec) < k:
                continue
            total += 1
            sym_vec = np.dot(sym_vec, vec)
            sym = ts(*sym_vec)
            sym.add_site_family(lat)
    assert total > 20


def test_translational_symmetry_reversed():
    np.random.seed(30)
    lat = lattice.general(np.identity(3))
    sites = [lat(i, j, k) for i in range(-2, 6) for j in range(-2, 6)
                          for k in range(-2, 6)]
    for i in range(4):
            periods = np.random.randint(-5, 5, (3, 3))
            try:
                sym = lattice.TranslationalSymmetry(*periods)
                rsym = sym.reversed()
                for site in sites:
                    assert_equal(sym.to_fd(site), rsym.to_fd(site))
                    assert_equal(sym.which(site), -rsym.which(site))
                    vec = np.array([1, 1, 1])
                    assert_equal(sym.act(vec, site), rsym.act(-vec, site))
            except ValueError:
                pass


def test_monatomic_lattice():
    lat = lattice.square()
    lat2 = lattice.general(np.identity(2))
    lat3 = lattice.square(name='no')
    assert len(set([lat, lat2, lat3, lat(0, 0), lat2(0, 0), lat3(0, 0)])) == 4



# Tutorial 2.6.2. "Lattice description": using different lattices
# ===============================================================
#
# Physics background
# ------------------
# - conductance of a NS-junction (Andreev reflection, superconducting gap)
#
# Kwant features highlighted
# --------------------------
# - Implementing electron and hole ("orbital") degrees of freedom
#   using different lattices

import kwant

# For plotting
from matplotlib import pyplot


#HIDDEN_BEGIN_zuuw
def make_system(a=1, W=10, L=10, barrier=1.5, barrierpos=(3, 4),
                mu=0.4, Delta=0.1, Deltapos=4, t=1.0):
    # Start with an empty tight-binding system and two square lattices,
    # corresponding to electron and hole degree of freedom
    lat_e = kwant.lattice.square(a, name='e')
    lat_h = kwant.lattice.square(a, name='h')
#HIDDEN_END_zuuw

#HIDDEN_BEGIN_pqmp
    sys = kwant.Builder()

    #### Define the scattering region. ####
    sys[(lat_e(x, y) for x in range(L) for y in range(W))] = 4 * t - mu
    sys[(lat_h(x, y) for x in range(L) for y in range(W))] = mu - 4 * t

    # the tunnel barrier
    sys[(lat_e(x, y) for x in range(barrierpos[0], barrierpos[1])
         for y in range(W))] = 4 * t + barrier - mu
    sys[(lat_h(x, y) for x in range(barrierpos[0], barrierpos[1])
         for y in range(W))] = mu - 4 * t - barrier

    # hoppings for both electrons and holes
    sys[lat_e.neighbors()] = -t
    sys[lat_h.neighbors()] = t

    # Superconducting order parameter enters as hopping between
    # electrons and holes
    sys[((lat_e(x, y), lat_h(x, y)) for x in range(Deltapos, L)
         for y in range(W))] = Delta
#HIDDEN_END_pqmp

    #### Define the leads. ####
#HIDDEN_BEGIN_ttth
    # Symmetry for the left leads.
    sym_left = kwant.TranslationalSymmetry((-a, 0))

    # left electron lead
    lead0 = kwant.Builder(sym_left)
    lead0[(lat_e(0, j) for j in xrange(W))] = 4 * t - mu
    lead0[lat_e.neighbors()] = -t

    # left hole lead
    lead1 = kwant.Builder(sym_left)
    lead1[(lat_h(0, j) for j in xrange(W))] = mu - 4 * t
    lead1[lat_h.neighbors()] = t
#HIDDEN_END_ttth

    # Then the lead to the right
    # this one is superconducting and thus is comprised of electrons
    # AND holes
#HIDDEN_BEGIN_mhiw
    sym_right = kwant.TranslationalSymmetry((a, 0))
    lead2 = kwant.Builder(sym_right)
    lead2 += lead0
    lead2 += lead1
    lead2[((lat_e(0, j), lat_h(0, j)) for j in xrange(W))] = Delta
#HIDDEN_END_mhiw

    #### Attach the leads and return the system. ####
#HIDDEN_BEGIN_ozsr
    sys.attach_lead(lead0)
    sys.attach_lead(lead1)
    sys.attach_lead(lead2)

    return sys
#HIDDEN_END_ozsr


#HIDDEN_BEGIN_jbjt
def plot_conductance(sys, energies):
    # Compute conductance
    data = []
    for energy in energies:
        smatrix = kwant.smatrix(sys, energy)
        # Conductance is N - R_ee + R_he
        data.append(smatrix.submatrix(0, 0).shape[0] -
                    smatrix.transmission(0, 0) +
                    smatrix.transmission(1, 0))
#HIDDEN_END_jbjt

    pyplot.figure()
    pyplot.plot(energies, data)
    pyplot.xlabel("energy [t]")
    pyplot.ylabel("conductance [e^2/h]")
    pyplot.show()


def main():
    sys = make_system()

    # Check that the system looks as intended.
    kwant.plot(sys)

    # Finalize the system.
    sys = sys.finalized()

    plot_conductance(sys, energies=[0.002 * i for i in xrange(100)])


# Call the main function if the script gets executed (as opposed to imported).
# See <http://docs.python.org/library/__main__.html>.
if __name__ == '__main__':
    main()
